@@ -19,6 +19,7 @@ package org.apache.celeborn.client;
 
 import java.io.IOException;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.atomic.LongAdder;
 
 import org.apache.hadoop.fs.FileSystem;
 import org.slf4j.Logger;
@@ -37,10 +38,12 @@ import org.apache.celeborn.common.write.PushState;
  * implementation
  */
 public abstract class ShuffleClient {
+  private static Logger logger = LoggerFactory.getLogger(ShuffleClient.class);
   private static volatile ShuffleClient _instance;
   private static volatile boolean initialized = false;
   private static volatile FileSystem hdfsFs;
-  private static Logger logger = LoggerFactory.getLogger(ShuffleClient.class);
+  private static LongAdder totalReadCounter = new LongAdder();
+  private static LongAdder localShuffleReadCounter = new LongAdder();
 
   // for testing
   public static void reset() {
@@ -56,8 +59,7 @@ public abstract class ShuffleClient {
       String driverHost,
       int port,
       CelebornConf conf,
-      UserIdentifier userIdentifier,
-      boolean isDriver) {
+      UserIdentifier userIdentifier) {
     if (null == _instance || !initialized) {
       synchronized (ShuffleClient.class) {
         if (null == _instance) {
@@ -67,12 +69,12 @@ public abstract class ShuffleClient {
           // be
           // assigned. An Executor will only construct a ShuffleClient singleton once. At this time,
           // when communicating with LifecycleManager, it will cause a NullPointerException.
-          _instance = new ShuffleClientImpl(appUniqueId, conf, userIdentifier, isDriver);
+          _instance = new ShuffleClientImpl(appUniqueId, conf, userIdentifier);
           _instance.setupLifecycleManagerRef(driverHost, port);
           initialized = true;
         } else if (!initialized) {
           _instance.shutdown();
-          _instance = new ShuffleClientImpl(appUniqueId, conf, userIdentifier, isDriver);
+          _instance = new ShuffleClientImpl(appUniqueId, conf, userIdentifier);
           _instance.setupLifecycleManagerRef(driverHost, port);
           initialized = true;
         }
@@ -96,6 +98,23 @@ public abstract class ShuffleClient {
     return hdfsFs;
   }
 
+  public static void incrementLocalReadCounter() {
+    localShuffleReadCounter.increment();
+    totalReadCounter.increment();
+  }
+
+  public static void incrementTotalReadCounter() {
+    totalReadCounter.increment();
+  }
+
+  public static String getReadCounters() {
+    long totalReadCount = totalReadCounter.longValue();
+    long localReadCount = localShuffleReadCounter.longValue();
+    return String.format(
+        "Current client read %d(local)/%d(total) partitions, local ratio %.2f",
+        localReadCount, totalReadCount, (localReadCount * 1.0d / totalReadCount) * 100);
+  }
+
   public abstract void setupLifecycleManagerRef(String host, int port);
 
   public abstract void setupLifecycleManagerRef(RpcEndpointRef endpointRef);
@@ -150,7 +169,7 @@ public abstract class ShuffleClient {
   public abstract CelebornInputStream readPartition(
       int shuffleId, int partitionId, int attemptNumber) throws IOException;
 
-  public abstract boolean unregisterShuffle(int shuffleId, boolean isDriver);
+  public abstract boolean cleanupShuffle(int shuffleId);
 
   public abstract void shutdown();
 
