@@ -20,7 +20,10 @@ package org.apache.celeborn.client;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.*;
-import java.util.concurrent.*;
+import java.util.concurrent.Callable;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.TimeUnit;
 
 import scala.reflect.ClassTag$;
 
@@ -34,7 +37,6 @@ import org.slf4j.LoggerFactory;
 import org.apache.celeborn.client.compress.Compressor;
 import org.apache.celeborn.client.read.CelebornInputStream;
 import org.apache.celeborn.common.CelebornConf;
-import org.apache.celeborn.common.client.MasterClient;
 import org.apache.celeborn.common.exception.CelebornIOException;
 import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.network.TransportContext;
@@ -110,8 +112,6 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   protected final String appUniqueId;
 
-  private ScheduledExecutorService heartbeater;
-
   private ThreadLocal<Compressor> compressorThreadLocal =
       new ThreadLocal<Compressor>() {
         @Override
@@ -153,8 +153,7 @@ public class ShuffleClientImpl extends ShuffleClient {
   protected final Map<Integer, ReduceFileGroups> reduceFileGroupsMap =
       JavaUtils.newConcurrentHashMap();
 
-  public ShuffleClientImpl(
-      String appUniqueId, CelebornConf conf, UserIdentifier userIdentifier, boolean isDriver) {
+  public ShuffleClientImpl(String appUniqueId, CelebornConf conf, UserIdentifier userIdentifier) {
     super();
     this.appUniqueId = appUniqueId;
     this.conf = conf;
@@ -177,7 +176,7 @@ public class ShuffleClientImpl extends ShuffleClient {
 
     String module = TransportModuleConstants.DATA_MODULE;
     TransportConf dataTransportConf =
-        Utils.fromCelebornConf(conf, module, conf.getInt("celeborn" + module + ".io.threads", 8));
+        Utils.fromCelebornConf(conf, module, conf.getInt("celeborn." + module + ".io.threads", 8));
     TransportContext context =
         new TransportContext(
             dataTransportConf, new BaseMessageHandler(), conf.clientCloseIdleConnections());
@@ -193,25 +192,6 @@ public class ShuffleClientImpl extends ShuffleClient {
             "celeborn-shuffle-split", pushSplitPartitionThreads, 60);
     reviveManager = new ReviveManager(this, conf);
 
-    if (!isDriver) {
-      heartbeater =
-          ThreadUtils.newDaemonSingleThreadScheduledExecutor(
-              "celeborn-lifecyclemanager-heartbeater");
-      heartbeater.scheduleAtFixedRate(
-          () -> {
-            PbHeartbeatFromClientResponse resp =
-                lifecycleManagerRef.askSync(
-                    HeartbeatFromClient$.MODULE$.apply(reducePartitionMap.keySet()),
-                    ClassTag$.MODULE$.apply(PbHeartbeatFromClientResponse.class));
-            List<Integer> unknownShuffleIds = resp.getUnknownShuffleIdList();
-            for (int i = 0; i < unknownShuffleIds.size(); i++) {
-              unregisterShuffle(unknownShuffleIds.get(i), false);
-            }
-          },
-          conf.clientHeartbeatToLifecycleManagerInterval(),
-          conf.clientHeartbeatToLifecycleManagerInterval(),
-          TimeUnit.MILLISECONDS);
-    }
     logger.info("Created ShuffleClientImpl, appUniqueId: {}", appUniqueId);
   }
 
@@ -236,7 +216,7 @@ public class ShuffleClientImpl extends ShuffleClient {
       int shuffleId,
       byte[] body,
       int batchId,
-      RpcResponseCallback wrappedCallback,
+      PushDataRpcResponseCallback pushDataRpcResponseCallback,
       PushState pushState,
       ReviveRequest request,
       int remainReviveTimes,
@@ -270,7 +250,7 @@ public class ShuffleClientImpl extends ShuffleClient {
           loc);
       pushState.removeBatch(batchId, loc.hostAndPushPort());
     } else if (request.reviveStatus != StatusCode.SUCCESS.getValue()) {
-      wrappedCallback.onFailure(
+      pushDataRpcResponseCallback.onFailure(
           new CelebornIOException(
               cause
                   + " then revive but "
@@ -293,7 +273,7 @@ public class ShuffleClientImpl extends ShuffleClient {
           batchId,
           newLoc);
       try {
-        if (!isPushTargetWorkerExcluded(newLoc, wrappedCallback)) {
+        if (!isPushTargetWorkerExcluded(newLoc, pushDataRpcResponseCallback)) {
           if (!testRetryRevive || remainReviveTimes < 1) {
             TransportClient client =
                 dataClientFactory.createClient(newLoc.getHost(), newLoc.getPushPort(), partitionId);
@@ -301,7 +281,8 @@ public class ShuffleClientImpl extends ShuffleClient {
             String shuffleKey = Utils.makeShuffleKey(appUniqueId, shuffleId);
             PushData newPushData =
                 new PushData(PRIMARY_MODE, shuffleKey, newLoc.getUniqueId(), newBuffer);
-            client.pushData(newPushData, pushDataTimeout, wrappedCallback);
+            pushDataRpcResponseCallback.updateLatestPartition(newLoc);
+            client.pushData(newPushData, pushDataTimeout, pushDataRpcResponseCallback);
           } else {
             throw new RuntimeException(
                 "Mock push data submit retry failed. remainReviveTimes = "
@@ -319,7 +300,7 @@ public class ShuffleClientImpl extends ShuffleClient {
             batchId,
             newLoc,
             e);
-        wrappedCallback.onFailure(
+        pushDataRpcResponseCallback.onFailure(
             new CelebornIOException(StatusCode.PUSH_DATA_CREATE_CONNECTION_FAIL_PRIMARY, e));
       }
     }
@@ -624,7 +605,7 @@ public class ShuffleClientImpl extends ShuffleClient {
    * @param shuffleMap
    * @param partitionId
    * @param epoch
-   * @param wait wheter to wait for some time for a newer PartitionLocation
+   * @param wait whether to wait for some time for a newer PartitionLocation
    * @return
    */
   boolean newerPartitionLocationExists(
@@ -694,7 +675,9 @@ public class ShuffleClientImpl extends ShuffleClient {
           attemptId,
           partitionId);
       return true;
-    } else if (results == null || results.get(partitionId) != StatusCode.SUCCESS.getValue()) {
+    } else if (results == null
+        || !results.containsKey(partitionId)
+        || results.get(partitionId) != StatusCode.SUCCESS.getValue()) {
       return false;
     } else {
       return true;
@@ -748,7 +731,7 @@ public class ShuffleClientImpl extends ShuffleClient {
             pushExcludedWorkers.remove(loc.getPeer().hostAndPushPort());
           }
         } else if (StatusCode.STAGE_ENDED.getValue() == statusCode) {
-          stageEnded(shuffleId);
+          stageEndShuffleSet.add(shuffleId);
           return results;
         } else if (StatusCode.SHUFFLE_NOT_REGISTERED.getValue() == statusCode) {
           logger.error("SHUFFLE_NOT_REGISTERED!");
@@ -776,6 +759,10 @@ public class ShuffleClientImpl extends ShuffleClient {
     }
   }
 
+  private interface PushDataRpcResponseCallback extends RpcResponseCallback {
+    default void updateLatestPartition(PartitionLocation latest) {}
+  }
+
   public int pushOrMergeData(
       int shuffleId,
       int mapId,
@@ -889,7 +876,6 @@ public class ShuffleClientImpl extends ShuffleClient {
           new RpcResponseCallback() {
             @Override
             public void onSuccess(ByteBuffer response) {
-              pushState.removeBatch(nextBatchId, loc.hostAndPushPort());
               if (response.remaining() > 0 && response.get() == StatusCode.STAGE_ENDED.getValue()) {
                 stageEndShuffleSet.add(shuffleId);
               }
@@ -914,8 +900,16 @@ public class ShuffleClientImpl extends ShuffleClient {
           };
 
       RpcResponseCallback wrappedCallback =
-          new RpcResponseCallback() {
+          new PushDataRpcResponseCallback() {
             int remainReviveTimes = maxReviveTimes;
+            PartitionLocation latest = loc;
+
+            @Override
+            public void updateLatestPartition(PartitionLocation latest) {
+              pushState.addBatch(nextBatchId, latest.hostAndPushPort());
+              pushState.removeBatch(nextBatchId, this.latest.hostAndPushPort());
+              this.latest = latest;
+            }
 
             @Override
             public void onSuccess(ByteBuffer response) {
@@ -924,19 +918,20 @@ public class ShuffleClientImpl extends ShuffleClient {
                 if (reason == StatusCode.SOFT_SPLIT.getValue()) {
                   logger.debug(
                       "Push data to {} soft split required for shuffle {} map {} attempt {} partition {} batch {}.",
-                      loc.hostAndPushPort(),
+                      latest.hostAndPushPort(),
                       shuffleId,
                       mapId,
                       attemptId,
                       partitionId,
                       nextBatchId);
-                  splitPartition(shuffleId, partitionId, loc);
-                  pushState.onSuccess(loc.hostAndPushPort());
+                  splitPartition(shuffleId, partitionId, latest);
+                  pushState.onSuccess(latest.hostAndPushPort());
+                  pushState.removeBatch(nextBatchId, latest.hostAndPushPort());
                   callback.onSuccess(response);
                 } else if (reason == StatusCode.HARD_SPLIT.getValue()) {
                   logger.debug(
                       "Push data to {} hard split required for shuffle {} map {} attempt {} partition {} batch {}.",
-                      loc.hostAndPushPort(),
+                      latest.hostAndPushPort(),
                       shuffleId,
                       mapId,
                       attemptId,
@@ -948,8 +943,8 @@ public class ShuffleClientImpl extends ShuffleClient {
                           mapId,
                           attemptId,
                           partitionId,
-                          loc.getEpoch(),
-                          loc,
+                          latest.getEpoch(),
+                          latest,
                           StatusCode.HARD_SPLIT);
                   reviveManager.addRequest(reviveRequest);
                   long dueTime =
@@ -971,33 +966,37 @@ public class ShuffleClientImpl extends ShuffleClient {
                 } else if (reason == StatusCode.PUSH_DATA_SUCCESS_PRIMARY_CONGESTED.getValue()) {
                   logger.debug(
                       "Push data to {} primary congestion required for shuffle {} map {} attempt {} partition {} batch {}.",
-                      loc.hostAndPushPort(),
+                      latest.hostAndPushPort(),
                       shuffleId,
                       mapId,
                       attemptId,
                       partitionId,
                       nextBatchId);
-                  pushState.onCongestControl(loc.hostAndPushPort());
+                  pushState.onCongestControl(latest.hostAndPushPort());
+                  pushState.removeBatch(nextBatchId, latest.hostAndPushPort());
                   callback.onSuccess(response);
                 } else if (reason == StatusCode.PUSH_DATA_SUCCESS_REPLICA_CONGESTED.getValue()) {
                   logger.debug(
                       "Push data to {} replica congestion required for shuffle {} map {} attempt {} partition {} batch {}.",
-                      loc.hostAndPushPort(),
+                      latest.hostAndPushPort(),
                       shuffleId,
                       mapId,
                       attemptId,
                       partitionId,
                       nextBatchId);
-                  pushState.onCongestControl(loc.hostAndPushPort());
+                  pushState.onCongestControl(latest.hostAndPushPort());
+                  pushState.removeBatch(nextBatchId, latest.hostAndPushPort());
                   callback.onSuccess(response);
                 } else {
                   // StageEnd.
                   response.rewind();
-                  pushState.onSuccess(loc.hostAndPushPort());
+                  pushState.onSuccess(latest.hostAndPushPort());
+                  pushState.removeBatch(nextBatchId, latest.hostAndPushPort());
                   callback.onSuccess(response);
                 }
               } else {
-                pushState.onSuccess(loc.hostAndPushPort());
+                pushState.onSuccess(latest.hostAndPushPort());
+                pushState.removeBatch(nextBatchId, latest.hostAndPushPort());
                 callback.onSuccess(response);
               }
             }
@@ -1021,7 +1020,7 @@ public class ShuffleClientImpl extends ShuffleClient {
 
               logger.error(
                   "Push data to {} failed for shuffle {} map {} attempt {} partition {} batch {}, remain revive times {}.",
-                  loc.hostAndPushPort(),
+                  latest.hostAndPushPort(),
                   shuffleId,
                   mapId,
                   attemptId,
@@ -1034,7 +1033,7 @@ public class ShuffleClientImpl extends ShuffleClient {
                 remainReviveTimes = remainReviveTimes - 1;
                 ReviveRequest reviveRequest =
                     new ReviveRequest(
-                        shuffleId, mapId, attemptId, partitionId, loc.getEpoch(), loc, cause);
+                        shuffleId, mapId, attemptId, partitionId, latest.getEpoch(), latest, cause);
                 reviveManager.addRequest(reviveRequest);
                 long dueTime =
                     System.currentTimeMillis()
@@ -1053,10 +1052,10 @@ public class ShuffleClientImpl extends ShuffleClient {
                             remainReviveTimes,
                             dueTime));
               } else {
-                pushState.removeBatch(nextBatchId, loc.hostAndPushPort());
+                pushState.removeBatch(nextBatchId, latest.hostAndPushPort());
                 logger.info(
                     "Push data to {} failed but mapper already ended for shuffle {} map {} attempt {} partition {} batch {}, remain revive times {}.",
-                    loc.hostAndPushPort(),
+                    latest.hostAndPushPort(),
                     shuffleId,
                     mapId,
                     attemptId,
@@ -1516,18 +1515,7 @@ public class ShuffleClientImpl extends ShuffleClient {
   }
 
   @Override
-  public boolean unregisterShuffle(int shuffleId, boolean isDriver) {
-    if (isDriver) {
-      try {
-        lifecycleManagerRef.send(
-            UnregisterShuffle$.MODULE$.apply(appUniqueId, shuffleId, MasterClient.genRequestId()));
-      } catch (Exception e) {
-        // If some exceptions need to be ignored, they shouldn't be logged as error-level,
-        // otherwise it will mislead users.
-        logger.error("Send UnregisterShuffle failed, ignore.", e);
-      }
-    }
-
+  public boolean cleanupShuffle(int shuffleId) {
     // clear status
     reducePartitionMap.remove(shuffleId);
     reduceFileGroupsMap.remove(shuffleId);
@@ -1566,7 +1554,7 @@ public class ShuffleClientImpl extends ShuffleClient {
           logger.info(
               "Shuffle {} request reducer file group success using {} ms, result partition size {}.",
               shuffleId,
-              (System.nanoTime() - getReducerFileGroupStartTime) / 1000_000,
+              TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - getReducerFileGroupStartTime),
               response.fileGroup().size());
           return new ReduceFileGroups(
               response.fileGroup(), response.attempts(), response.partitionIds());
@@ -1652,9 +1640,6 @@ public class ShuffleClientImpl extends ShuffleClient {
     if (null != lifecycleManagerRef) {
       lifecycleManagerRef = null;
     }
-    if (null != heartbeater) {
-      heartbeater.shutdown();
-    }
     pushExcludedWorkers.clear();
     fetchExcludedWorkers.clear();
     logger.warn("Shuffle client has been shutdown!");
@@ -1673,10 +1658,10 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   boolean mapperEnded(int shuffleId, int mapId) {
     return (mapperEndMap.containsKey(shuffleId) && mapperEndMap.get(shuffleId).contains(mapId))
-        || stageEnded(shuffleId);
+        || isStageEnded(shuffleId);
   }
 
-  protected boolean stageEnded(int shuffleId) {
+  protected boolean isStageEnded(int shuffleId) {
     return stageEndShuffleSet.contains(shuffleId);
   }
 
