@@ -20,7 +20,6 @@ package org.apache.celeborn.client
 import java.util
 import java.util.{function, List => JList}
 import java.util.concurrent.{ConcurrentHashMap, ScheduledFuture, TimeUnit}
-import java.util.function.Predicate
 
 import scala.collection.JavaConverters._
 import scala.collection.mutable
@@ -59,6 +58,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
   private val lifecycleHost = Utils.localHostName(conf)
 
   private val shuffleExpiredCheckIntervalMs = conf.shuffleExpiredCheckIntervalMs
+  private val slotsAssignMaxWorkers = conf.clientSlotAssignMaxWorkers
   private val pushReplicateEnabled = conf.clientPushReplicateEnabled
   private val pushRackAwareEnabled = conf.clientReserveSlotsRackAwareEnabled
   private val partitionSplitThreshold = conf.shufflePartitionSplitThreshold
@@ -198,11 +198,6 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     case StageEnd(shuffleId) =>
       logInfo(s"Received StageEnd request, shuffleId $shuffleId.")
       handleStageEnd(shuffleId)
-    case pb: PbUnregisterShuffle =>
-      val shuffleId = pb.getShuffleId
-      logDebug(s"Received UnregisterShuffle request," +
-        s"shuffleId $shuffleId.")
-      handleUnregisterShuffle(shuffleId)
   }
 
   override def receiveAndReply(context: RpcCallContext): PartialFunction[Any, Unit] = {
@@ -301,9 +296,6 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       logDebug(s"Received GetShuffleFileGroup request," +
         s"shuffleId $shuffleId.")
       handleGetReducerFileGroup(context, shuffleId)
-
-    case pb: PbHeartbeatFromClient =>
-      handleHeartbeatFromClient(context, pb)
   }
 
   private def offerAndReserveSlots(
@@ -407,20 +399,19 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     // First, request to get allocated slots from Primary
     val ids = new util.ArrayList[Integer](numPartitions)
     (0 until numPartitions).foreach(idx => ids.add(Integer.valueOf(idx)))
-    val res = requestPrimaryRequestSlotsWithRetry(shuffleId, ids)
+    val res = requestMasterRequestSlotsWithRetry(shuffleId, ids)
 
     res.status match {
       case StatusCode.REQUEST_FAILED =>
-        logDebug(s"OfferSlots RPC request failed for $shuffleId!")
+        logInfo(s"OfferSlots RPC request failed for $shuffleId!")
         reply(RegisterShuffleResponse(StatusCode.REQUEST_FAILED, Array.empty))
         return
       case StatusCode.SLOT_NOT_AVAILABLE =>
-        logDebug(s"OfferSlots for $shuffleId failed!")
+        logInfo(s"OfferSlots for $shuffleId failed!")
         reply(RegisterShuffleResponse(StatusCode.SLOT_NOT_AVAILABLE, Array.empty))
         return
       case StatusCode.SUCCESS =>
-        logInfo(s"OfferSlots for $shuffleId Success!")
-        logDebug(s" Slots Info: ${res.workerResource}")
+        logInfo(s"OfferSlots for $shuffleId Success!Slots Info: ${res.workerResource}")
       case _ => // won't happen
         throw new UnsupportedOperationException()
     }
@@ -433,7 +424,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
 
     // Second, for each worker, try to initialize the endpoint.
     val parallelism = Math.min(Math.max(1, slots.size()), conf.clientRpcMaxParallelism)
-    ThreadUtils.parmap(slots.asScala.to, "InitWorkerRef", parallelism) { case (workerInfo, _) =>
+    ThreadUtils.parmap(slots.asScala, "InitWorkerRef", parallelism) { case (workerInfo, _) =>
       try {
         workerInfo.endpoint =
           rpcEnv.setupEndpointRef(RpcAddress.apply(workerInfo.host, workerInfo.rpcPort), WORKER_EP)
@@ -465,12 +456,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     if (!reserveSlotsSuccess) {
       logError(s"reserve buffer for $shuffleId failed, reply to all.")
       reply(RegisterShuffleResponse(StatusCode.RESERVE_SLOTS_FAILED, Array.empty))
-      // tell Primary to release slots
-      requestPrimaryReleaseSlots(
-        ReleaseSlots(appUniqueId, shuffleId, List.empty.asJava, List.empty.asJava))
     } else {
-      logInfo(s"ReserveSlots for $shuffleId success!")
-      logDebug(s"Allocated Slots: $slots")
+      logInfo(s"ReserveSlots for $shuffleId success with details:$slots!")
       // Forth, register shuffle success, update status
       val allocatedWorkers =
         JavaUtils.newConcurrentHashMap[WorkerInfo, ShufflePartitionLocationInfo]()
@@ -512,7 +499,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         false)
       return
     }
-
+    logDebug(
+      s"[handleRevive] shuffle $shuffleId, $mapIds, $partitionIds, $oldEpochs, $oldPartitions, $causes")
     if (commitManager.isStageEnd(shuffleId)) {
       logError(s"[handleRevive] shuffle $shuffleId stage ended!")
       contextWrapper.reply(
@@ -568,17 +556,6 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     commitManager.handleGetReducerFileGroup(context, shuffleId)
   }
 
-  private def handleHeartbeatFromClient(
-      context: RpcCallContext,
-      pb: PbHeartbeatFromClient): Unit = {
-    val shuffleIds = new util.ArrayList[Integer](pb.getShuffleIdList)
-    val pred = new Predicate[Integer] {
-      override def test(id: Integer): Boolean = registeredShuffle.contains(id)
-    }
-    shuffleIds.removeIf(pred)
-    context.reply(HeartbeatFromClientResponse(shuffleIds))
-  }
-
   private def handleStageEnd(shuffleId: Int): Unit = {
     // check whether shuffle has registered
     if (!registeredShuffle.contains(shuffleId)) {
@@ -596,8 +573,6 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         partitionLocationInfo.removeAllPrimaryPartitions()
         partitionLocationInfo.removeAllReplicaPartitions()
       }
-      requestPrimaryReleaseSlots(
-        ReleaseSlots(appUniqueId, shuffleId, List.empty.asJava, List.empty.asJava))
     }
   }
 
@@ -631,8 +606,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     reply(mapperAttemptFinishedSuccess)
   }
 
-  def handleUnregisterShuffle(
-      shuffleId: Int): Unit = {
+  def unregisterShuffle(shuffleId: Int): Unit = {
     if (getPartitionType(shuffleId) == PartitionType.REDUCE) {
       // if StageEnd has not been handled, trigger StageEnd
       if (!commitManager.isStageEnd(shuffleId)) {
@@ -649,13 +623,6 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       }
     }
 
-    if (shuffleResourceExists(shuffleId)) {
-      logWarning(s"Partition exists for shuffle $shuffleId, " +
-        "maybe caused by task rerun or speculative.")
-      requestPrimaryReleaseSlots(
-        ReleaseSlots(appUniqueId, shuffleId, List.empty.asJava, List.empty.asJava))
-    }
-
     // add shuffleKey to delay shuffle removal set
     unregisterShuffleTime.put(shuffleId, System.currentTimeMillis())
 
@@ -682,7 +649,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     val workerPartitionLocations = slots.asScala.filter(p => !p._2._1.isEmpty || !p._2._2.isEmpty)
     val parallelism =
       Math.min(Math.max(1, workerPartitionLocations.size), conf.clientRpcMaxParallelism)
-    ThreadUtils.parmap(workerPartitionLocations.to, "ReserveSlot", parallelism) {
+    ThreadUtils.parmap(workerPartitionLocations, "ReserveSlot", parallelism) {
       case (workerInfo, (primaryLocations, replicaLocations)) =>
         val res = requestWorkerReserveSlots(
           workerInfo.endpoint,
@@ -696,7 +663,10 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
             getPartitionType(shuffleId),
             rangeReadFilter,
             userIdentifier,
-            conf.pushDataTimeoutMs))
+            conf.pushDataTimeoutMs,
+            if (getPartitionType(shuffleId) == PartitionType.MAP)
+              conf.clientShuffleMapPartitionSplitEnabled
+            else true))
         if (res.status.equals(StatusCode.SUCCESS)) {
           logDebug(s"Successfully allocated " +
             s"partitions buffer for shuffleId $shuffleId" +
@@ -783,19 +753,6 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       destroySlotsWithRetry(shuffleId, destroyResource)
       logInfo(s"Destroyed peer partitions for reserve buffer failed workers " +
         s"shuffleId $shuffleId, $destroyResource")
-
-      val workerIds = new util.ArrayList[String]()
-      val workerSlotsPerDisk = new util.ArrayList[util.Map[String, Integer]]()
-      Utils.getSlotsPerDisk(destroyResource).asScala.foreach {
-        case (workerInfo, slotsPerDisk) =>
-          workerIds.add(workerInfo.toUniqueId())
-          workerSlotsPerDisk.add(slotsPerDisk)
-      }
-      val msg = ReleaseSlots(appUniqueId, shuffleId, workerIds, workerSlotsPerDisk)
-      requestPrimaryReleaseSlots(msg)
-      logDebug(s"Released slots for reserve buffer failed workers " +
-        s"${workerIds.asScala.mkString(",")}" + s"${slots.asScala.mkString(",")}" +
-        s"shuffleId $shuffleId")
     }
   }
 
@@ -1039,7 +996,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         latestPartitionLocation.remove(shuffleId)
         commitManager.removeExpiredShuffle(shuffleId)
         changePartitionManager.removeExpiredShuffle(shuffleId)
-        val unregisterShuffleResponse = requestPrimaryUnregisterShuffle(
+        val unregisterShuffleResponse = requestMasterUnregisterShuffle(
           UnregisterShuffle(appUniqueId, shuffleId, MasterClient.genRequestId()))
         // if unregister shuffle not success, wait next turn
         if (StatusCode.SUCCESS == Utils.toStatusCode(unregisterShuffleResponse.getStatus)) {
@@ -1049,7 +1006,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     }
   }
 
-  private def requestPrimaryRequestSlotsWithRetry(
+  private def requestMasterRequestSlotsWithRetry(
       shuffleId: Int,
       ids: util.ArrayList[Integer]): RequestSlotsResponse = {
     val req =
@@ -1060,16 +1017,17 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         lifecycleHost,
         pushReplicateEnabled,
         pushRackAwareEnabled,
-        userIdentifier)
-    val res = requestPrimaryRequestSlots(req)
+        userIdentifier,
+        slotsAssignMaxWorkers)
+    val res = requestMasterRequestSlots(req)
     if (res.status != StatusCode.SUCCESS) {
-      requestPrimaryRequestSlots(req)
+      requestMasterRequestSlots(req)
     } else {
       res
     }
   }
 
-  private def requestPrimaryRequestSlots(message: RequestSlots): RequestSlotsResponse = {
+  private def requestMasterRequestSlots(message: RequestSlots): RequestSlotsResponse = {
     val shuffleKey = Utils.makeShuffleKey(message.applicationId, message.shuffleId)
     try {
       masterClient.askSync[RequestSlotsResponse](message, classOf[RequestSlotsResponse])
@@ -1110,17 +1068,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     }
   }
 
-  private def requestPrimaryReleaseSlots(message: ReleaseSlots): ReleaseSlotsResponse = {
-    try {
-      masterClient.askSync[ReleaseSlotsResponse](message, classOf[ReleaseSlotsResponse])
-    } catch {
-      case e: Exception =>
-        logError(s"AskSync ReleaseSlots for ${message.shuffleId} failed.", e)
-        ReleaseSlotsResponse(StatusCode.REQUEST_FAILED)
-    }
-  }
-
-  private def requestPrimaryUnregisterShuffle(message: PbUnregisterShuffle)
+  private def requestMasterUnregisterShuffle(message: PbUnregisterShuffle)
       : PbUnregisterShuffleResponse = {
     try {
       masterClient.askSync[PbUnregisterShuffleResponse](
@@ -1146,9 +1094,17 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     }
   }
 
-  private def shuffleResourceExists(shuffleId: Int): Boolean = {
-    val workerPartitionInfos = workerSnapshots(shuffleId)
-    workerPartitionInfos != null && workerPartitionInfos.values().asScala.exists(!_.isEmpty())
+  def checkWorkersAvailable(): PbCheckWorkersAvailableResponse = {
+    try {
+      masterClient.askSync[PbCheckWorkersAvailableResponse](
+        CheckWorkersAvailable(),
+        classOf[PbCheckWorkersAvailableResponse])
+    } catch {
+      case e: Exception =>
+        val msg = s"AskSync Cluster check workers available for $userIdentifier failed."
+        logError(msg, e)
+        CheckWorkersAvailableResponse(false)
+    }
   }
 
   // Once a partition is released, it will be never needed anymore
