@@ -120,14 +120,14 @@ public class TransportClient implements Closeable {
       int len,
       long fetchDataTimeout,
       ChunkReceivedCallback callback) {
+
+    StreamChunkSlice streamChunkSlice = new StreamChunkSlice(streamId, chunkIndex, offset, len);
     if (logger.isDebugEnabled()) {
       logger.debug(
           "Sending fetch chunk request {} to {}.",
-          chunkIndex,
+          streamChunkSlice,
           NettyUtils.getRemoteAddress(channel));
     }
-
-    StreamChunkSlice streamChunkSlice = new StreamChunkSlice(streamId, chunkIndex, offset, len);
     StdChannelListener listener =
         new StdChannelListener(streamChunkSlice) {
           @Override
@@ -170,6 +170,22 @@ public class TransportClient implements Closeable {
     return requestId;
   }
 
+  /**
+   * Sends an opaque message to the RpcHandler on the server-side.
+   *
+   * @param message The message to send.
+   * @return The RPC's id.
+   */
+  public long sendRpc(ByteBuffer message) {
+    if (logger.isTraceEnabled()) {
+      logger.trace("Sending RPC to {}", NettyUtils.getRemoteAddress(channel));
+    }
+
+    long requestId = requestId();
+    channel.writeAndFlush(new RpcRequest(requestId, new NioManagedBuffer(message)));
+    return requestId;
+  }
+
   public ChannelFuture pushData(
       PushData pushData, long pushDataTimeout, RpcResponseCallback callback) {
     return pushData(pushData, pushDataTimeout, callback, null);
@@ -306,9 +322,9 @@ public class TransportClient implements Closeable {
     @Override
     public void operationComplete(Future<? super Void> future) throws Exception {
       if (future.isSuccess()) {
-        if (logger.isTraceEnabled()) {
+        if (logger.isDebugEnabled()) {
           long timeTaken = System.currentTimeMillis() - startTime;
-          logger.trace(
+          logger.debug(
               "Sending request {} to {} took {} ms",
               requestId,
               NettyUtils.getRemoteAddress(channel),
