@@ -23,6 +23,7 @@ import java.net.InetSocketAddress;
 import java.net.SocketAddress;
 import java.util.Random;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
 
 import com.google.common.base.Preconditions;
@@ -120,7 +121,7 @@ public class TransportClientFactory implements Closeable {
       throws IOException, InterruptedException {
     // Get connection from the connection pool first.
     // If it is not found or not active, create a new one.
-    // Use unresolved address here to avoid DNS resolution each time we creates a client.
+    // Use unresolved address here to avoid DNS resolution each time we create a client.
     final InetSocketAddress unresolvedAddress =
         InetSocketAddress.createUnresolved(remoteHost, remotePort);
 
@@ -147,8 +148,11 @@ public class TransportClientFactory implements Closeable {
       }
 
       if (cachedClient.isActive()) {
-        logger.trace(
-            "Returning cached connection to {}: {}", cachedClient.getSocketAddress(), cachedClient);
+        logger.debug(
+            "Returning cached connection from {} to {}: {}",
+            cachedClient.getChannel().localAddress(),
+            cachedClient.getSocketAddress(),
+            cachedClient);
         return cachedClient;
       }
     }
@@ -157,11 +161,15 @@ public class TransportClientFactory implements Closeable {
     // Multiple threads might race here to create new connections. Keep only one of them active.
     final long preResolveHost = System.nanoTime();
     final InetSocketAddress resolvedAddress = new InetSocketAddress(remoteHost, remotePort);
-    final long hostResolveTimeMs = (System.nanoTime() - preResolveHost) / 1000000;
+    final long hostResolveTimeMs =
+        TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - preResolveHost);
+    final String resolveMsg = resolvedAddress.isUnresolved() ? "failed" : "succeed";
     if (hostResolveTimeMs > 2000) {
-      logger.warn("DNS resolution for {} took {} ms", resolvedAddress, hostResolveTimeMs);
+      logger.warn(
+          "DNS resolution {} for {} took {} ms", resolveMsg, resolvedAddress, hostResolveTimeMs);
     } else {
-      logger.trace("DNS resolution for {} took {} ms", resolvedAddress, hostResolveTimeMs);
+      logger.trace(
+          "DNS resolution {} for {} took {} ms", resolveMsg, resolvedAddress, hostResolveTimeMs);
     }
 
     synchronized (clientPool.locks[clientIndex]) {
@@ -169,7 +177,11 @@ public class TransportClientFactory implements Closeable {
 
       if (cachedClient != null) {
         if (cachedClient.isActive()) {
-          logger.trace("Returning cached connection to {}: {}", resolvedAddress, cachedClient);
+          logger.debug(
+              "Returning cached connection from {} to {}: {}",
+              cachedClient.getChannel().localAddress(),
+              resolvedAddress,
+              cachedClient);
           return cachedClient;
         } else {
           logger.info("Found inactive connection to {}, creating a new one.", resolvedAddress);
@@ -237,7 +249,8 @@ public class TransportClientFactory implements Closeable {
     TransportClient client = clientRef.get();
     assert client != null : "Channel future completed successfully with null client";
 
-    logger.debug("Connection to {} successful", address);
+    logger.debug(
+        "Connection from {} to {} successful", client.getChannel().localAddress(), address);
 
     return client;
   }
