@@ -52,6 +52,8 @@ import org.apache.celeborn.common.CelebornConf;
  */
 public final class ShutdownHookManager {
 
+  private static final Logger logger = LoggerFactory.getLogger(ShutdownHookManager.class);
+
   private static final ShutdownHookManager MGR = new ShutdownHookManager();
 
   private static final Logger LOG = LoggerFactory.getLogger(ShutdownHookManager.class);
@@ -105,23 +107,17 @@ public final class ShutdownHookManager {
     for (HookEntry entry : MGR.getShutdownHooksInOrder()) {
       Future<?> future = EXECUTOR.submit(entry.getHook());
       try {
+        logger.info(
+            "timeout {}",
+            Utils.msDurationToString(
+                entry.getTimeUnit().convert(entry.getTimeout(), TimeUnit.MILLISECONDS)));
         future.get(entry.getTimeout(), entry.getTimeUnit());
       } catch (TimeoutException ex) {
         timeouts++;
         future.cancel(true);
-        LOG.warn(
-            "ShutdownHook '"
-                + entry.getHook().getClass().getSimpleName()
-                + "' timeout, "
-                + ex.toString(),
-            ex);
+        LOG.warn("ShutdownHook '{}' timeout", entry.getHook().getClass().getSimpleName(), ex);
       } catch (Throwable ex) {
-        LOG.warn(
-            "ShutdownHook '"
-                + entry.getHook().getClass().getSimpleName()
-                + "' failed, "
-                + ex.toString(),
-            ex);
+        LOG.warn("ShutdownHook '{}' failed", entry.getHook().getClass().getSimpleName(), ex);
       }
     }
     return timeouts;
@@ -138,13 +134,13 @@ public final class ShutdownHookManager {
       long shutdownTimeout = getShutdownTimeout(conf);
       if (!EXECUTOR.awaitTermination(shutdownTimeout, TIME_UNIT_DEFAULT)) {
         // timeout waiting for the
-        LOG.error("ShutdownHookManger shutdown forcefully after" + " {} seconds.", shutdownTimeout);
+        LOG.error("ShutdownHookManger shutdown forcefully after {} seconds.", shutdownTimeout);
         EXECUTOR.shutdownNow();
       }
       LOG.debug("ShutdownHookManger completed shutdown.");
     } catch (InterruptedException ex) {
       // interrupted.
-      LOG.error("ShutdownHookManger interrupted while waiting for " + "termination.", ex);
+      LOG.error("ShutdownHookManger interrupted while waiting for termination.", ex);
       EXECUTOR.shutdownNow();
       Thread.currentThread().interrupt();
     }
@@ -165,8 +161,8 @@ public final class ShutdownHookManager {
   static class HookEntry {
     private final Runnable hook;
     private final int priority;
-    private final long timeout;
-    private final TimeUnit unit;
+    private long timeout;
+    private TimeUnit unit;
 
     HookEntry(Runnable hook, int priority) {
       this(hook, priority, getShutdownTimeout(new CelebornConf()), TIME_UNIT_DEFAULT);
@@ -207,6 +203,11 @@ public final class ShutdownHookManager {
       return timeout;
     }
 
+    public void setTimeout(long timeout, TimeUnit unit) {
+      this.timeout = timeout;
+      this.unit = unit;
+    }
+
     TimeUnit getTimeUnit() {
       return unit;
     }
@@ -279,6 +280,10 @@ public final class ShutdownHookManager {
     hooks.add(new HookEntry(shutdownHook, priority, timeout, unit));
   }
 
+  public void updateTimeout(long timeout, TimeUnit unit) {
+    hooks.forEach(hook -> hook.setTimeout(timeout, unit));
+  }
+
   /**
    * Removes a shutdownHook.
    *
@@ -296,7 +301,7 @@ public final class ShutdownHookManager {
    * Indicates if a shutdownHook is registered or not.
    *
    * @param shutdownHook shutdownHook to check if registered.
-   * @return TRUE/FALSE depending if the shutdownHook is is registered.
+   * @return TRUE/FALSE depending if the shutdownHook is registered.
    */
   public boolean hasShutdownHook(Runnable shutdownHook) {
     return hooks.contains(new HookEntry(shutdownHook, 0));
