@@ -66,6 +66,8 @@ object ControlMessages extends Logging {
 
   case object CheckForApplicationTimeOut extends Message
 
+  case object CheckForWorkerUnavailableInfoTimeout extends Message
+
   case object CheckForHDFSExpiredDirsTimeout extends Message
 
   case object RemoveExpiredShuffle extends Message
@@ -113,6 +115,7 @@ object ControlMessages extends Logging {
       userResourceConsumption: util.Map[UserIdentifier, ResourceConsumption],
       activeShuffleKeys: util.Set[String],
       estimatedAppDiskUsage: util.HashMap[String, java.lang.Long],
+      highWorkload: Boolean,
       override var requestId: String = ZERO_UUID) extends MasterRequestMessage
 
   case class HeartbeatFromWorkerResponse(
@@ -166,9 +169,12 @@ object ControlMessages extends Logging {
       shouldReplicate: Boolean,
       shouldRackAware: Boolean,
       userIdentifier: UserIdentifier,
+      maxWorkers: Int,
       override var requestId: String = ZERO_UUID)
     extends MasterRequestMessage
 
+  // Keep it for compatible reason
+  @deprecated
   case class ReleaseSlots(
       applicationId: String,
       shuffleId: Int,
@@ -177,6 +183,8 @@ object ControlMessages extends Logging {
       override var requestId: String = ZERO_UUID)
     extends MasterRequestMessage
 
+  // Keep it for compatible reason
+  @deprecated
   case class ReleaseSlotsResponse(status: StatusCode)
     extends MasterMessage
 
@@ -244,23 +252,6 @@ object ControlMessages extends Logging {
     }
   }
 
-  object HeartbeatFromClient {
-    def apply(
-        shuffleIds: util.Set[Integer]): PbHeartbeatFromClient = {
-      PbHeartbeatFromClient.newBuilder()
-        .addAllShuffleId(shuffleIds)
-        .build()
-    }
-  }
-
-  object HeartbeatFromClientResponse {
-    def apply(shuflfeIds: util.List[Integer]): PbHeartbeatFromClientResponse = {
-      PbHeartbeatFromClientResponse.newBuilder()
-        .addAllUnknownShuffleId(shuflfeIds)
-        .build()
-    }
-  }
-
   case class MapperEnd(
       shuffleId: Int,
       mapId: Int,
@@ -365,6 +356,31 @@ object ControlMessages extends Logging {
       unavailable: util.List[WorkerInfo],
       override var requestId: String = ZERO_UUID) extends MasterRequestMessage
 
+  object CheckWorkersAvailable {
+    def apply(): PbCheckWorkersAvailable = {
+      PbCheckWorkersAvailable.newBuilder().build()
+    }
+  }
+
+  object CheckWorkersAvailableResponse {
+    def apply(isAvailable: Boolean): PbCheckWorkersAvailableResponse =
+      PbCheckWorkersAvailableResponse.newBuilder()
+        .setAvailable(isAvailable)
+        .build()
+  }
+
+  object RemoveWorkersUnavailableInfo {
+    def apply(
+        unavailable: util.List[WorkerInfo],
+        requestId: String): PbRemoveWorkersUnavailableInfo =
+      PbRemoveWorkersUnavailableInfo.newBuilder()
+        .setRequestId(requestId)
+        .addAllWorkerInfo(unavailable.asScala.map { workerInfo =>
+          PbSerDeUtils.toPbWorkerInfo(workerInfo, true)
+        }.toList.asJava)
+        .build()
+  }
+
   /**
    * ==========================================
    *         handled by worker
@@ -388,7 +404,8 @@ object ControlMessages extends Logging {
       partitionType: PartitionType,
       rangeReadFilter: Boolean,
       userIdentifier: UserIdentifier,
-      pushDataTimeout: Long)
+      pushDataTimeout: Long,
+      partitionSplitEnabled: Boolean = false)
     extends WorkerMessage
 
   case class ReserveSlotsResponse(
@@ -464,6 +481,7 @@ object ControlMessages extends Logging {
           userResourceConsumption,
           activeShuffleKeys,
           estimatedAppDiskUsage,
+          highWorkload,
           requestId) =>
       val pbDisks = disks.map(PbSerDeUtils.toPbDiskInfo).asJava
       val pbUserResourceConsumption =
@@ -478,6 +496,7 @@ object ControlMessages extends Logging {
         .setReplicatePort(replicatePort)
         .addAllActiveShuffleKeys(activeShuffleKeys)
         .putAllEstimatedAppDiskUsage(estimatedAppDiskUsage)
+        .setHighWorkload(highWorkload)
         .setRequestId(requestId)
         .build().toByteArray
       new TransportMessage(MessageType.HEARTBEAT_FROM_WORKER, payload)
@@ -506,6 +525,7 @@ object ControlMessages extends Logging {
           shouldReplicate,
           shouldRackAware,
           userIdentifier,
+          maxWorkers,
           requestId) =>
       val payload = PbRequestSlots.newBuilder()
         .setApplicationId(applicationId)
@@ -514,6 +534,7 @@ object ControlMessages extends Logging {
         .setHostname(hostname)
         .setShouldReplicate(shouldReplicate)
         .setShouldRackAware(shouldRackAware)
+        .setMaxWorkers(maxWorkers)
         .setRequestId(requestId)
         .setUserIdentifier(PbSerDeUtils.toPbUserIdentifier(userIdentifier))
         .build().toByteArray
@@ -552,12 +573,6 @@ object ControlMessages extends Logging {
     case pb: PbChangeLocationResponse =>
       new TransportMessage(MessageType.CHANGE_LOCATION_RESPONSE, pb.toByteArray)
 
-    case pb: PbHeartbeatFromClient =>
-      new TransportMessage(MessageType.HEARTBEAT_FROM_CLIENT, pb.toByteArray)
-
-    case pb: PbHeartbeatFromClientResponse =>
-      new TransportMessage(MessageType.HEARTBEAT_FROM_CLIENT_RESPONSE, pb.toByteArray)
-
     case MapperEnd(shuffleId, mapId, attemptId, numMappers, partitionId) =>
       val payload = PbMapperEnd.newBuilder()
         .setShuffleId(shuffleId)
@@ -707,6 +722,9 @@ object ControlMessages extends Logging {
         .setRequestId(requestId).build().toByteArray
       new TransportMessage(MessageType.REPORT_WORKER_FAILURE, payload)
 
+    case pb: PbRemoveWorkersUnavailableInfo =>
+      new TransportMessage(MessageType.REMOVE_WORKERS_UNAVAILABLE_INFO, pb.toByteArray)
+
     case pb: PbRegisterWorkerResponse =>
       new TransportMessage(MessageType.REGISTER_WORKER_RESPONSE, pb.toByteArray)
 
@@ -720,7 +738,8 @@ object ControlMessages extends Logging {
           partType,
           rangeReadFilter,
           userIdentifier,
-          pushDataTimeout) =>
+          pushDataTimeout,
+          partitionSplitEnabled) =>
       val payload = PbReserveSlots.newBuilder()
         .setApplicationId(applicationId)
         .setShuffleId(shuffleId)
@@ -734,6 +753,7 @@ object ControlMessages extends Logging {
         .setRangeReadFilter(rangeReadFilter)
         .setUserIdentifier(PbSerDeUtils.toPbUserIdentifier(userIdentifier))
         .setPushDataTimeout(pushDataTimeout)
+        .setPartitionSplitEnabled(partitionSplitEnabled)
         .build().toByteArray
       new TransportMessage(MessageType.RESERVE_SLOTS, payload)
 
@@ -804,6 +824,12 @@ object ControlMessages extends Logging {
 
     case OneWayMessageResponse =>
       new TransportMessage(MessageType.ONE_WAY_MESSAGE_RESPONSE, null)
+
+    case pb: PbCheckWorkersAvailable =>
+      new TransportMessage(MessageType.CHECK_WORKERS_AVAILABLE, pb.toByteArray)
+
+    case pb: PbCheckWorkersAvailableResponse =>
+      new TransportMessage(MessageType.CHECK_WORKERS_AVAILABLE_RESPONSE, pb.toByteArray)
   }
 
   // TODO change return type to GeneratedMessageV3
@@ -820,6 +846,22 @@ object ControlMessages extends Logging {
         logError(msg)
         throw new UnsupportedOperationException(msg)
 
+      // keep it for compatible reason
+      case RELEASE_SLOTS_VALUE =>
+        val pbReleaseSlots = PbReleaseSlots.parseFrom(message.getPayload)
+        val slotsList = pbReleaseSlots.getSlotsList.asScala.map(pbSlot =>
+          new util.HashMap[String, Integer](pbSlot.getSlotMap)).toList.asJava
+        ReleaseSlots(
+          pbReleaseSlots.getApplicationId,
+          pbReleaseSlots.getShuffleId,
+          new util.ArrayList[String](pbReleaseSlots.getWorkerIdsList),
+          new util.ArrayList[util.Map[String, Integer]](slotsList),
+          pbReleaseSlots.getRequestId)
+
+      case RELEASE_SLOTS_RESPONSE_VALUE =>
+        val pbReleaseSlotsResponse = PbReleaseSlotsResponse.parseFrom(message.getPayload)
+        ReleaseSlotsResponse(Utils.toStatusCode(pbReleaseSlotsResponse.getStatus))
+
       case REGISTER_WORKER_VALUE =>
         PbRegisterWorker.parseFrom(message.getPayload)
 
@@ -828,7 +870,8 @@ object ControlMessages extends Logging {
         val estimatedAppDiskUsage = new util.HashMap[String, java.lang.Long]()
         val userResourceConsumption = PbSerDeUtils.fromPbUserResourceConsumption(
           pbHeartbeatFromWorker.getUserResourceConsumptionMap)
-        val pbDisks = pbHeartbeatFromWorker.getDisksList.asScala.map(PbSerDeUtils.fromPbDiskInfo)
+        val pbDisks =
+          pbHeartbeatFromWorker.getDisksList.asScala.toSeq.map(PbSerDeUtils.fromPbDiskInfo)
         if (!pbHeartbeatFromWorker.getEstimatedAppDiskUsageMap.isEmpty) {
           estimatedAppDiskUsage.putAll(pbHeartbeatFromWorker.getEstimatedAppDiskUsageMap)
         }
@@ -846,6 +889,7 @@ object ControlMessages extends Logging {
           userResourceConsumption,
           activeShuffleKeys,
           estimatedAppDiskUsage,
+          pbHeartbeatFromWorker.getHighWorkload,
           pbHeartbeatFromWorker.getRequestId)
 
       case HEARTBEAT_RESPONSE_VALUE =>
@@ -877,23 +921,9 @@ object ControlMessages extends Logging {
           pbRequestSlots.getShouldReplicate,
           pbRequestSlots.getShouldRackAware,
           userIdentifier,
+          pbRequestSlots.getMaxWorkers,
           pbRequestSlots.getRequestId)
 
-      case RELEASE_SLOTS_VALUE =>
-        val pbReleaseSlots = PbReleaseSlots.parseFrom(message.getPayload)
-        val slotsList = pbReleaseSlots.getSlotsList.asScala.map(pbSlot =>
-          new util.HashMap[String, Integer](pbSlot.getSlotMap)).toList.asJava
-        ReleaseSlots(
-          pbReleaseSlots.getApplicationId,
-          pbReleaseSlots.getShuffleId,
-          new util.ArrayList[String](pbReleaseSlots.getWorkerIdsList),
-          new util.ArrayList[util.Map[String, Integer]](slotsList),
-          pbReleaseSlots.getRequestId)
-
-      case RELEASE_SLOTS_RESPONSE_VALUE =>
-        val pbReleaseSlotsResponse = PbReleaseSlotsResponse.parseFrom(message.getPayload)
-        ReleaseSlotsResponse(Utils.toStatusCode(pbReleaseSlotsResponse.getStatus))
-
       case REQUEST_SLOTS_RESPONSE_VALUE =>
         val pbRequestSlotsResponse = PbRequestSlotsResponse.parseFrom(message.getPayload)
         RequestSlotsResponse(
@@ -907,12 +937,6 @@ object ControlMessages extends Logging {
       case CHANGE_LOCATION_RESPONSE_VALUE =>
         PbChangeLocationResponse.parseFrom(message.getPayload)
 
-      case HEARTBEAT_FROM_CLIENT_VALUE =>
-        PbHeartbeatFromClient.parseFrom(message.getPayload)
-
-      case HEARTBEAT_FROM_CLIENT_RESPONSE_VALUE =>
-        PbHeartbeatFromClientResponse.parseFrom(message.getPayload)
-
       case MAPPER_END_VALUE =>
         val pbMapperEnd = PbMapperEnd.parseFrom(message.getPayload)
         MapperEnd(
@@ -1021,6 +1045,9 @@ object ControlMessages extends Logging {
             .asScala.map(PbSerDeUtils.fromPbWorkerInfo).toList.asJava),
           pbReportWorkerUnavailable.getRequestId)
 
+      case REMOVE_WORKERS_UNAVAILABLE_INFO_VALUE =>
+        PbRemoveWorkersUnavailableInfo.parseFrom(message.getPayload)
+
       case REGISTER_WORKER_RESPONSE_VALUE =>
         PbRegisterWorkerResponse.parseFrom(message.getPayload)
 
@@ -1039,7 +1066,8 @@ object ControlMessages extends Logging {
           Utils.toPartitionType(pbReserveSlots.getPartitionType),
           pbReserveSlots.getRangeReadFilter,
           userIdentifier,
-          pbReserveSlots.getPushDataTimeout)
+          pbReserveSlots.getPushDataTimeout,
+          pbReserveSlots.getPartitionSplitEnabled)
 
       case RESERVE_SLOTS_RESPONSE_VALUE =>
         val pbReserveSlotsResponse = PbReserveSlotsResponse.parseFrom(message.getPayload)
@@ -1126,6 +1154,12 @@ object ControlMessages extends Logging {
       case STAGE_END_RESPONSE_VALUE =>
         val pbStageEndResponse = PbStageEndResponse.parseFrom(message.getPayload)
         StageEndResponse(Utils.toStatusCode(pbStageEndResponse.getStatus))
+
+      case CHECK_WORKERS_AVAILABLE_VALUE =>
+        PbCheckWorkersAvailable.parseFrom(message.getPayload)
+
+      case CHECK_WORKERS_AVAILABLE_RESPONSE_VALUE =>
+        PbCheckWorkersAvailableResponse.parseFrom(message.getPayload)
     }
   }
 }
