@@ -20,8 +20,6 @@ package org.apache.celeborn.common.util
 import java.util.concurrent._
 import java.util.concurrent.{ForkJoinPool => SForkJoinPool, ForkJoinWorkerThread => SForkJoinWorkerThread}
 
-import scala.collection.TraversableLike
-import scala.collection.generic.CanBuildFrom
 import scala.concurrent.{Awaitable, ExecutionContext, ExecutionContextExecutor, Future}
 import scala.concurrent.duration.{Duration, FiniteDuration}
 import scala.language.higherKinds
@@ -287,13 +285,7 @@ object ThreadUtils {
    * @return new collection in which each element was given from the input collection `in` by
    *         applying the lambda function `f`.
    */
-  def parmap[I, O, Col[X] <: TraversableLike[X, Col[X]]](
-      in: Col[I],
-      prefix: String,
-      maxThreads: Int)(f: I => O)(implicit
-      cbf: CanBuildFrom[Col[I], Future[O], Col[Future[O]]], // For in.map
-      cbf2: CanBuildFrom[Col[Future[O]], O, Col[O]] // for Future.sequence
-  ): Col[O] = {
+  def parmap[I, O](in: Iterable[I], prefix: String, maxThreads: Int)(f: I => O): Iterable[O] = {
     val pool = newForkJoinPool(prefix, maxThreads)
     try {
       implicit val ec = ExecutionContext.fromExecutor(pool)
