@@ -25,7 +25,7 @@ import java.nio.ByteBuffer
 import java.nio.channels.FileChannel
 import java.nio.charset.StandardCharsets
 import java.util
-import java.util.{Locale, Properties, Random, UUID}
+import java.util.{ArrayList, List, Locale, Properties, Random, UUID}
 import java.util.concurrent.{Callable, ThreadPoolExecutor, TimeoutException, TimeUnit}
 
 import scala.annotation.tailrec
@@ -38,10 +38,11 @@ import scala.util.control.{ControlThrowable, NonFatal}
 import com.google.protobuf.{ByteString, GeneratedMessageV3}
 import io.netty.channel.unix.Errors.NativeIoException
 import org.apache.commons.lang3.SystemUtils
-import org.apache.hadoop.fs.Path
+import org.apache.hadoop.fs.{FSDataInputStream, Path}
 import org.roaringbitmap.RoaringBitmap
 
 import org.apache.celeborn.common.CelebornConf
+import org.apache.celeborn.common.CelebornConf.PORT_MAX_RETRY
 import org.apache.celeborn.common.exception.CelebornException
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.meta.{DiskStatus, FileInfo, WorkerInfo}
@@ -274,13 +275,12 @@ object Utils extends Logging {
                 s"${e.getMessage}: Service$serviceString failed after " +
                   s"$maxRetries retries (on a random free port)! " +
                   s"Consider explicitly setting the appropriate binding address for " +
-                  s"the service$serviceString (for example spark.driver.bindAddress " +
-                  s"for SparkDriver) to the correct binding address."
+                  s"the service$serviceString to the correct binding address."
               } else {
                 s"${e.getMessage}: Service$serviceString failed after " +
                   s"$maxRetries retries (starting from $startPort)! Consider explicitly setting " +
-                  s"the appropriate port for the service$serviceString (for example spark.ui.port " +
-                  s"for SparkUI) to an available port or increasing spark.port.maxRetries."
+                  s"the appropriate port for the service$serviceString to an available port " +
+                  s"or increasing ${PORT_MAX_RETRY.key}."
               }
             val exception = new BindException(exceptionMessage)
             // restore original stack trace
@@ -613,7 +613,7 @@ object Utils extends Logging {
   }
 
   def makeReducerKey(shuffleId: Int, partitionId: Int): String = {
-    s"shuffleId-$partitionId"
+    s"$shuffleId-$partitionId"
   }
 
   def makeMapKey(applicationId: String, shuffleId: Int, mapId: Int, attemptId: Int): String = {
@@ -649,14 +649,12 @@ object Utils extends Logging {
   }
 
   private def readProcessStdout(process: Process): String = {
-    val stream = process.getInputStream
-    val sb = new StringBuilder
-    var res = stream.read()
-    while (res != -1) {
-      sb.append(res.toChar)
-      res = stream.read()
+    val source = Source.fromInputStream(process.getInputStream)
+    try {
+      source.mkString
+    } finally {
+      source.close()
     }
-    sb.toString()
   }
 
   def runCommand(cmd: String): String = {
