@@ -50,7 +50,10 @@ public class SlotsAllocator {
 
   public static Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>>
       offerSlotsRoundRobin(
-          List<WorkerInfo> workers, List<Integer> partitionIds, boolean shouldReplicate) {
+          List<WorkerInfo> workers,
+          List<Integer> partitionIds,
+          boolean shouldReplicate,
+          boolean shouldRackAware) {
     if (partitionIds.isEmpty()) {
       return new HashMap<>();
     }
@@ -71,9 +74,14 @@ public class SlotsAllocator {
         }
       }
     }
-    List<Integer> remain = roundRobin(slots, partitionIds, workers, restrictions, shouldReplicate);
+    List<Integer> remain =
+        roundRobin(slots, partitionIds, workers, restrictions, shouldReplicate, shouldRackAware);
     if (!remain.isEmpty()) {
-      roundRobin(slots, remain, workers, null, shouldReplicate);
+      remain = roundRobin(slots, remain, workers, null, shouldReplicate, shouldRackAware);
+    }
+
+    if (!remain.isEmpty()) {
+      roundRobin(slots, remain, workers, null, shouldReplicate, false);
     }
     return slots;
   }
@@ -88,6 +96,7 @@ public class SlotsAllocator {
           List<WorkerInfo> workers,
           List<Integer> partitionIds,
           boolean shouldReplicate,
+          boolean shouldRackAware,
           long minimumUsableSize,
           int diskGroupCount,
           double diskGroupGradient,
@@ -123,7 +132,7 @@ public class SlotsAllocator {
       logger.warn(
           "offer slots for {} fallback to roundrobin because there is no usable disks",
           StringUtils.join(partitionIds, ','));
-      return offerSlotsRoundRobin(workers, partitionIds, shouldReplicate);
+      return offerSlotsRoundRobin(workers, partitionIds, shouldReplicate, shouldRackAware);
     }
 
     if (!initialized) {
@@ -144,11 +153,21 @@ public class SlotsAllocator {
             partitionIds,
             new ArrayList<>(restriction.keySet()),
             restriction,
-            shouldReplicate);
+            shouldReplicate,
+            shouldRackAware);
     if (!remainPartitions.isEmpty()) {
-      roundRobin(slots, remainPartitions, new ArrayList<>(workers), null, shouldReplicate);
+      remainPartitions =
+          roundRobin(
+              slots,
+              remainPartitions,
+              new ArrayList<>(workers),
+              null,
+              shouldReplicate,
+              shouldRackAware);
+    }
+    if (!remainPartitions.isEmpty()) {
+      roundRobin(slots, remainPartitions, new ArrayList<>(workers), null, shouldReplicate, false);
     }
-
     return slots;
   }
 
@@ -174,70 +193,87 @@ public class SlotsAllocator {
       List<Integer> partitionIds,
       List<WorkerInfo> workers,
       Map<WorkerInfo, List<UsableDiskInfo>> restrictions,
-      boolean shouldReplicate) {
-    // workerInfo -> (diskIndexForMaster, diskIndexForSlave)
-    Map<WorkerInfo, Integer> workerDiskIndexForMaster = new HashMap<>();
-    Map<WorkerInfo, Integer> workerDiskIndexForSlave = new HashMap<>();
+      boolean shouldReplicate,
+      boolean shouldRackAware) {
+    // workerInfo -> (diskIndexForPrimary, diskIndexForReplica)
+    Map<WorkerInfo, Integer> workerDiskIndexForPrimary = new HashMap<>();
+    Map<WorkerInfo, Integer> workerDiskIndexForReplica = new HashMap<>();
     List<Integer> partitionIdList = new ArrayList<>(partitionIds);
-    int masterIndex = rand.nextInt(workers.size());
+    int primaryIndex = rand.nextInt(workers.size());
     Iterator<Integer> iter = partitionIdList.iterator();
     outer:
     while (iter.hasNext()) {
-      int nextMasterInd = masterIndex;
+      int nextPrimayInd = primaryIndex;
 
       int partitionId = iter.next();
       StorageInfo storageInfo = new StorageInfo();
       if (restrictions != null) {
-        while (restrictions.get(workers.get(nextMasterInd)).stream()
-                .mapToLong(i -> i.usableSlots)
-                .sum()
-            <= 0) {
-          nextMasterInd = (nextMasterInd + 1) % workers.size();
-          if (nextMasterInd == masterIndex) {
+        while (!haveUsableSlots(restrictions, workers, nextPrimayInd)) {
+          nextPrimayInd = (nextPrimayInd + 1) % workers.size();
+          if (nextPrimayInd == primaryIndex) {
             break outer;
           }
         }
         storageInfo =
-            getStorageInfo(workers, nextMasterInd, restrictions, workerDiskIndexForMaster);
+            getStorageInfo(workers, nextPrimayInd, restrictions, workerDiskIndexForPrimary);
       }
-      PartitionLocation masterPartition =
-          createLocation(partitionId, workers.get(nextMasterInd), null, storageInfo, true);
+      PartitionLocation primaryPartition =
+          createLocation(partitionId, workers.get(nextPrimayInd), null, storageInfo, true);
 
       if (shouldReplicate) {
-        int nextSlaveInd = (nextMasterInd + 1) % workers.size();
+        int nextReplicaInd = (nextPrimayInd + 1) % workers.size();
         if (restrictions != null) {
-          while (restrictions.get(workers.get(nextSlaveInd)).stream()
-                  .mapToLong(i -> i.usableSlots)
-                  .sum()
-              <= 0) {
-            nextSlaveInd = (nextSlaveInd + 1) % workers.size();
-            if (nextSlaveInd == nextMasterInd) {
+          while (!haveUsableSlots(restrictions, workers, nextReplicaInd)
+              || !satisfyRackAware(shouldRackAware, workers, nextPrimayInd, nextReplicaInd)) {
+            nextReplicaInd = (nextReplicaInd + 1) % workers.size();
+            if (nextReplicaInd == nextPrimayInd) {
               break outer;
             }
           }
           storageInfo =
-              getStorageInfo(workers, nextSlaveInd, restrictions, workerDiskIndexForSlave);
+              getStorageInfo(workers, nextReplicaInd, restrictions, workerDiskIndexForReplica);
+        } else if (shouldRackAware) {
+          while (!satisfyRackAware(true, workers, nextPrimayInd, nextReplicaInd)) {
+            nextReplicaInd = (nextReplicaInd + 1) % workers.size();
+            if (nextReplicaInd == nextPrimayInd) {
+              break outer;
+            }
+          }
         }
-        PartitionLocation slavePartition =
+        PartitionLocation replicaPartition =
             createLocation(
-                partitionId, workers.get(nextSlaveInd), masterPartition, storageInfo, false);
-        masterPartition.setPeer(slavePartition);
+                partitionId, workers.get(nextReplicaInd), primaryPartition, storageInfo, false);
+        primaryPartition.setPeer(replicaPartition);
         Tuple2<List<PartitionLocation>, List<PartitionLocation>> locations =
             slots.computeIfAbsent(
-                workers.get(nextSlaveInd), v -> new Tuple2<>(new ArrayList<>(), new ArrayList<>()));
-        locations._2.add(slavePartition);
+                workers.get(nextReplicaInd),
+                v -> new Tuple2<>(new ArrayList<>(), new ArrayList<>()));
+        locations._2.add(replicaPartition);
       }
 
       Tuple2<List<PartitionLocation>, List<PartitionLocation>> locations =
           slots.computeIfAbsent(
-              workers.get(nextMasterInd), v -> new Tuple2<>(new ArrayList<>(), new ArrayList<>()));
-      locations._1.add(masterPartition);
-      masterIndex = (nextMasterInd + 1) % workers.size();
+              workers.get(nextPrimayInd), v -> new Tuple2<>(new ArrayList<>(), new ArrayList<>()));
+      locations._1.add(primaryPartition);
+      primaryIndex = (nextPrimayInd + 1) % workers.size();
       iter.remove();
     }
     return partitionIdList;
   }
 
+  private static boolean haveUsableSlots(
+      Map<WorkerInfo, List<UsableDiskInfo>> restrictions, List<WorkerInfo> workers, int index) {
+    return restrictions.get(workers.get(index)).stream().mapToLong(i -> i.usableSlots).sum() > 0;
+  }
+
+  private static boolean satisfyRackAware(
+      boolean shouldRackAware, List<WorkerInfo> workers, int primaryIndex, int nextReplicaInd) {
+    return !shouldRackAware
+        || !Objects.equals(
+            workers.get(primaryIndex).networkLocation(),
+            workers.get(nextReplicaInd).networkLocation());
+  }
+
   private static void initLoadAwareAlgorithm(int diskGroups, double diskGroupGradient) {
     taskAllocationRatio = new double[diskGroups];
     double totalAllocations = 0;
@@ -396,7 +432,7 @@ public class SlotsAllocator {
       WorkerInfo workerInfo,
       PartitionLocation peer,
       StorageInfo storageInfo,
-      boolean isMaster) {
+      boolean isPrimary) {
     return new PartitionLocation(
         partitionIndex,
         0,
@@ -405,7 +441,7 @@ public class SlotsAllocator {
         workerInfo.pushPort(),
         workerInfo.fetchPort(),
         workerInfo.replicatePort(),
-        isMaster ? PartitionLocation.Mode.MASTER : PartitionLocation.Mode.SLAVE,
+        isPrimary ? PartitionLocation.Mode.PRIMARY : PartitionLocation.Mode.REPLICA,
         peer,
         storageInfo,
         new RoaringBitmap());
