@@ -43,6 +43,7 @@ import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.meta.AppDiskUsageMetric;
 import org.apache.celeborn.common.meta.AppDiskUsageSnapShot;
 import org.apache.celeborn.common.meta.DiskInfo;
+import org.apache.celeborn.common.meta.DiskStatus;
 import org.apache.celeborn.common.meta.WorkerInfo;
 import org.apache.celeborn.common.protocol.PbSnapshotMetaInfo;
 import org.apache.celeborn.common.quota.ResourceConsumption;
@@ -95,29 +96,6 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     }
   }
 
-  public void updateReleaseSlotsMeta(String shuffleKey) {
-    updateReleaseSlotsMeta(shuffleKey, null, null);
-  }
-
-  public void updateReleaseSlotsMeta(
-      String shuffleKey, List<String> workerIds, List<Map<String, Integer>> slots) {
-    if (workerIds != null && !workerIds.isEmpty()) {
-      for (int i = 0; i < workerIds.size(); i++) {
-        String workerId = workerIds.get(i);
-        WorkerInfo worker = WorkerInfo.fromUniqueId(workerId);
-        for (WorkerInfo w : workers) {
-          if (w.equals(worker)) {
-            Map<String, Integer> slotToRelease = slots.get(i);
-            LOG.info("release slots for worker {}, to release: {}", w, slotToRelease);
-            w.releaseSlots(shuffleKey, slotToRelease);
-          }
-        }
-      }
-    } else {
-      workers.forEach(workerInfo -> workerInfo.releaseSlots(shuffleKey));
-    }
-  }
-
   public void updateUnregisterShuffleMeta(String shuffleKey) {
     registeredShuffle.remove(shuffleKey);
   }
@@ -157,6 +135,17 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     excludedWorkers.remove(worker);
   }
 
+  public void removeWorkersUnavailableInfoMeta(List<WorkerInfo> unavailableWorkers) {
+    synchronized (workers) {
+      for (WorkerInfo workerInfo : unavailableWorkers) {
+        if (lostWorkers.containsKey(workerInfo)) {
+          lostWorkers.remove(workerInfo);
+          shutdownWorkers.remove(workerInfo);
+        }
+      }
+    }
+  }
+
   public void updateWorkerHeartbeatMeta(
       String host,
       int rpcPort,
@@ -166,7 +155,8 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
       Map<String, DiskInfo> disks,
       Map<UserIdentifier, ResourceConsumption> userResourceConsumption,
       Map<String, Long> estimatedAppDiskUsage,
-      long time) {
+      long time,
+      boolean highWorkload) {
     WorkerInfo worker =
         new WorkerInfo(
             host, rpcPort, pushPort, fetchPort, replicatePort, disks, userResourceConsumption);
@@ -184,10 +174,13 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     }
     appDiskUsageMetric.update(estimatedAppDiskUsage);
     // If using HDFSONLY mode, workers with empty disks should not be put into excluded worker list.
-    if (!excludedWorkers.contains(worker) && (disks.isEmpty() && !conf.hasHDFSStorage())) {
+    long healthyDiskNum =
+        disks.values().stream().filter(s -> s.status().equals(DiskStatus.HEALTHY)).count();
+    if (!excludedWorkers.contains(worker)
+        && (((disks.isEmpty() || healthyDiskNum <= 0) && !conf.hasHDFSStorage()) || highWorkload)) {
       LOG.debug("Worker: {} num total slots is 0, add to excluded list", worker);
       excludedWorkers.add(worker);
-    } else if (availableSlots.get() > 0) {
+    } else if ((availableSlots.get() > 0 || conf.hasHDFSStorage()) && !highWorkload) {
       // only unblack if numSlots larger than 0
       excludedWorkers.remove(worker);
     }
@@ -210,9 +203,10 @@ public abstract class AbstractMetaManager implements IMetadataHandler {
     synchronized (workers) {
       if (!workers.contains(workerInfo)) {
         workers.add(workerInfo);
-        shutdownWorkers.remove(workerInfo);
-        lostWorkers.remove(workerInfo);
       }
+      shutdownWorkers.remove(workerInfo);
+      lostWorkers.remove(workerInfo);
+      excludedWorkers.remove(workerInfo);
     }
   }
 
