@@ -20,6 +20,7 @@ package org.apache.celeborn.service.deploy.master
 import java.io.IOException
 import java.net.BindException
 import java.util
+import java.util.Collections
 import java.util.concurrent.{ConcurrentHashMap, ScheduledFuture, TimeUnit}
 
 import scala.collection.JavaConverters._
@@ -93,12 +94,15 @@ private[celeborn] class Master(
     ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-forward-message-thread")
   private var checkForWorkerTimeOutTask: ScheduledFuture[_] = _
   private var checkForApplicationTimeOutTask: ScheduledFuture[_] = _
+  private var checkForUnavailableWorkerTimeOutTask: ScheduledFuture[_] = _
   private var checkForHDFSRemnantDirsTimeOutTask: ScheduledFuture[_] = _
   private val nonEagerHandler = ThreadUtils.newDaemonCachedThreadPool("master-noneager-handler", 64)
 
   // Config constants
   private val workerHeartbeatTimeoutMs = conf.workerHeartbeatTimeout
   private val appHeartbeatTimeoutMs = conf.appHeartbeatTimeoutMs
+  private val workerUnavailableInfoExpireTimeoutMs = conf.workerUnavailableInfoExpireTimeout
+
   private val hdfsExpireDirsTimeoutMS = conf.hdfsExpireDirsTimeoutMS
   private val hasHDFSStorage = conf.hasHDFSStorage
 
@@ -117,6 +121,7 @@ private[celeborn] class Master(
 
   private def diskReserveSize = conf.workerDiskReserveSize
 
+  private val slotsAssignMaxWorkers = conf.masterSlotAssignMaxWorkers
   private val slotsAssignLoadAwareDiskGroupNum = conf.masterSlotAssignLoadAwareDiskGroupNum
   private val slotsAssignLoadAwareDiskGroupGradient =
     conf.masterSlotAssignLoadAwareDiskGroupGradient
@@ -189,6 +194,16 @@ private[celeborn] class Master(
       appHeartbeatTimeoutMs / 2,
       TimeUnit.MILLISECONDS)
 
+    checkForUnavailableWorkerTimeOutTask = forwardMessageThread.scheduleAtFixedRate(
+      new Runnable {
+        override def run(): Unit = Utils.tryLogNonFatalError {
+          self.send(CheckForWorkerUnavailableInfoTimeout)
+        }
+      },
+      0,
+      workerUnavailableInfoExpireTimeoutMs / 2,
+      TimeUnit.MILLISECONDS)
+
     if (hasHDFSStorage) {
       checkForHDFSRemnantDirsTimeOutTask = forwardMessageThread.scheduleAtFixedRate(
         new Runnable {
@@ -208,6 +223,9 @@ private[celeborn] class Master(
     if (checkForWorkerTimeOutTask != null) {
       checkForWorkerTimeOutTask.cancel(true)
     }
+    if (checkForUnavailableWorkerTimeOutTask != null) {
+      checkForUnavailableWorkerTimeOutTask.cancel(true)
+    }
     if (checkForApplicationTimeOutTask != null) {
       checkForApplicationTimeOutTask.cancel(true)
     }
@@ -236,6 +254,8 @@ private[celeborn] class Master(
   override def receive: PartialFunction[Any, Unit] = {
     case _: PbCheckForWorkerTimeout =>
       executeWithLeaderChecker(null, timeoutDeadWorkers())
+    case CheckForWorkerUnavailableInfoTimeout =>
+      executeWithLeaderChecker(null, timeoutWorkerUnavailableInfos())
     case CheckForApplicationTimeOut =>
       executeWithLeaderChecker(null, timeoutDeadApplications())
     case CheckForHDFSExpiredDirsTimeout =>
@@ -251,6 +271,12 @@ private[celeborn] class Master(
       executeWithLeaderChecker(
         null,
         handleWorkerLost(null, host, rpcPort, pushPort, fetchPort, replicatePort, requestId))
+    case pb: PbRemoveWorkersUnavailableInfo =>
+      val unavailableWorkers = new util.ArrayList[WorkerInfo](pb.getWorkerInfoList
+        .asScala.map(PbSerDeUtils.fromPbWorkerInfo).toList.asJava)
+      executeWithLeaderChecker(
+        null,
+        handleRemoveWorkersUnavailableInfos(unavailableWorkers, pb.getRequestId))
   }
 
   override def receiveAndReply(context: RpcCallContext): PartialFunction[Any, Unit] = {
@@ -301,17 +327,14 @@ private[celeborn] class Master(
           userResourceConsumption,
           requestId))
 
-    case requestSlots @ RequestSlots(_, _, _, _, _, _, _, _) =>
+    case ReleaseSlots(_, _, _, _, _) =>
+      // keep it for compatible reason
+      context.reply(ReleaseSlotsResponse(StatusCode.SUCCESS))
+
+    case requestSlots @ RequestSlots(_, _, _, _, _, _, _, _, _) =>
       logTrace(s"Received RequestSlots request $requestSlots.")
       executeWithLeaderChecker(context, handleRequestSlots(context, requestSlots))
 
-    case ReleaseSlots(applicationId, shuffleId, workerIds, slots, requestId) =>
-      logTrace(s"Received ReleaseSlots request $requestId, $applicationId, $shuffleId," +
-        s"workers ${workerIds.asScala.mkString(",")}, slots ${slots.asScala.mkString(",")}")
-      executeWithLeaderChecker(
-        context,
-        handleReleaseSlots(context, applicationId, shuffleId, workerIds, slots, requestId))
-
     case pb: PbUnregisterShuffle =>
       val applicationId = pb.getAppId
       val shuffleId = pb.getShuffleId
@@ -338,6 +361,7 @@ private[celeborn] class Master(
           userResourceConsumption,
           activeShuffleKey,
           estimatedAppDiskUsage,
+          highWorkload,
           requestId) =>
       logDebug(s"Received heartbeat from" +
         s" worker $host:$rpcPort:$pushPort:$fetchPort:$replicatePort with $disks.")
@@ -354,6 +378,7 @@ private[celeborn] class Master(
           userResourceConsumption,
           activeShuffleKey,
           estimatedAppDiskUsage,
+          highWorkload,
           requestId))
 
     case ReportWorkerUnavailable(failedWorkers: util.List[WorkerInfo], requestId: String) =>
@@ -375,6 +400,9 @@ private[celeborn] class Master(
 
     case CheckQuota(userIdentifier) =>
       executeWithLeaderChecker(context, handleCheckQuota(userIdentifier, context))
+
+    case _: PbCheckWorkersAvailable =>
+      executeWithLeaderChecker(context, handleCheckWorkersAvailable(context))
   }
 
   private def timeoutDeadWorkers() {
@@ -402,6 +430,25 @@ private[celeborn] class Master(
     }
   }
 
+  private def timeoutWorkerUnavailableInfos(): Unit = {
+    val currentTime = System.currentTimeMillis()
+    // Need increase timeout deadline to avoid long time leader election period
+    if (HAHelper.getWorkerTimeoutDeadline(statusSystem) > currentTime) {
+      return
+    }
+
+    val unavailableInfoTimeoutWorkers = lostWorkersSnapshot.asScala.filter {
+      case (_, lostTime) => currentTime - lostTime > workerUnavailableInfoExpireTimeoutMs
+    }.keySet.toList.asJava
+
+    if (!unavailableInfoTimeoutWorkers.isEmpty) {
+      logDebug(s"Remove unavailable info for workers: $unavailableInfoTimeoutWorkers")
+      self.send(RemoveWorkersUnavailableInfo(
+        unavailableInfoTimeoutWorkers,
+        MasterClient.genRequestId()));
+    }
+  }
+
   private def timeoutDeadApplications(): Unit = {
     val currentTime = System.currentTimeMillis()
     // Need increase timeout deadline to avoid long time leader election period
@@ -436,6 +483,7 @@ private[celeborn] class Master(
       userResourceConsumption: util.Map[UserIdentifier, ResourceConsumption],
       activeShuffleKeys: util.Set[String],
       estimatedAppDiskUsage: util.HashMap[String, java.lang.Long],
+      highWorkload: Boolean,
       requestId: String): Unit = {
     val targetWorker = new WorkerInfo(host, rpcPort, pushPort, fetchPort, replicatePort)
     val registered = workersSnapShot.asScala.contains(targetWorker)
@@ -453,6 +501,7 @@ private[celeborn] class Master(
         userResourceConsumption,
         estimatedAppDiskUsage,
         System.currentTimeMillis(),
+        highWorkload,
         requestId)
     }
 
@@ -520,7 +569,9 @@ private[celeborn] class Master(
     if (workersSnapShot.contains(workerToRegister)) {
       logWarning(s"Receive RegisterWorker while worker" +
         s" ${workerToRegister.toString()} already exists, re-register.")
+      // TODO: remove `WorkerRemove` because we have improve register logic to cover `WorkerRemove`
       statusSystem.handleWorkerRemove(host, rpcPort, pushPort, fetchPort, replicatePort, requestId)
+      val newRequestId = MasterClient.genRequestId()
       statusSystem.handleRegisterWorker(
         host,
         rpcPort,
@@ -529,7 +580,7 @@ private[celeborn] class Master(
         replicatePort,
         disks,
         userResourceConsumption,
-        requestId)
+        newRequestId)
       context.reply(RegisterWorkerResponse(true, "Worker in snapshot, re-register."))
     } else if (statusSystem.workerLostEvents.contains(workerToRegister)) {
       logWarning(s"Receive RegisterWorker while worker $workerToRegister " +
@@ -564,7 +615,15 @@ private[celeborn] class Master(
     val numReducers = requestSlots.partitionIdList.size()
     val shuffleKey = Utils.makeShuffleKey(requestSlots.applicationId, requestSlots.shuffleId)
 
-    val availableWorkers = workersAvailable()
+    var availableWorkers = workersAvailable()
+    Collections.shuffle(availableWorkers)
+    val numWorkers = Math.min(
+      Math.max(
+        if (requestSlots.shouldReplicate) 2 else 1,
+        if (requestSlots.maxWorkers <= 0) slotsAssignMaxWorkers
+        else Math.min(slotsAssignMaxWorkers, requestSlots.maxWorkers)),
+      availableWorkers.size())
+    availableWorkers = availableWorkers.subList(0, numWorkers)
     // offer slots
     val slots =
       masterSource.sample(MasterSource.OFFER_SLOTS_TIME, s"offerSlots-${Random.nextInt()}") {
@@ -630,17 +689,6 @@ private[celeborn] class Master(
     context.reply(RequestSlotsResponse(StatusCode.SUCCESS, slots.asInstanceOf[WorkerResource]))
   }
 
-  def handleReleaseSlots(
-      context: RpcCallContext,
-      applicationId: String,
-      shuffleId: Int,
-      workerIds: util.List[String],
-      slots: util.List[util.Map[String, Integer]],
-      requestId: String): Unit = {
-    // For compatibility, ignore this message
-    context.reply(ReleaseSlotsResponse(StatusCode.SUCCESS))
-  }
-
   def handleUnregisterShuffle(
       context: RpcCallContext,
       applicationId: String,
@@ -686,7 +734,13 @@ private[celeborn] class Master(
 
   private def checkAndCleanExpiredAppDirsOnHDFS(expiredDir: String = ""): Unit = {
     if (hadoopFs == null) {
-      hadoopFs = CelebornHadoopUtils.getHadoopFS(conf)
+      try {
+        hadoopFs = CelebornHadoopUtils.getHadoopFS(conf)
+      } catch {
+        case e: Exception =>
+          logError("Celeborn initialize HDFS failed.", e)
+          throw e
+      }
     }
     val hdfsWorkPath = new Path(conf.hdfsDir, conf.workerWorkingDir)
     if (hadoopFs.exists(hdfsWorkPath)) {
@@ -733,6 +787,12 @@ private[celeborn] class Master(
     }
   }
 
+  private def handleRemoveWorkersUnavailableInfos(
+      unavailableWorkers: util.List[WorkerInfo],
+      requestId: String): Unit = {
+    statusSystem.handleRemoveWorkersUnavailableInfo(unavailableWorkers, requestId);
+  }
+
   private def computeUserResourceConsumption(userIdentifier: UserIdentifier)
       : ResourceConsumption = {
     val current = System.currentTimeMillis()
@@ -780,6 +840,10 @@ private[celeborn] class Master(
     context.reply(CheckQuotaResponse(isAvailable, reason))
   }
 
+  private def handleCheckWorkersAvailable(context: RpcCallContext): Unit = {
+    context.reply(CheckWorkersAvailableResponse(!workersAvailable().isEmpty))
+  }
+
   private def workersAvailable(
       tmpExcludedWorkerList: Set[WorkerInfo] = Set.empty): util.List[WorkerInfo] = {
     workersSnapShot.asScala.filter { w =>
@@ -894,11 +958,11 @@ private[celeborn] class Master(
     rpcEnv.awaitTermination()
   }
 
-  override def close(): Unit = synchronized {
+  override def stop(exitKind: Int): Unit = synchronized {
     if (!stopped) {
       logInfo("Stopping Master")
-      stop()
-      super.close()
+      rpcEnv.stop(self)
+      super.stop(exitKind)
       logInfo("Master stopped.")
       stopped = true
     }
@@ -909,7 +973,13 @@ private[deploy] object Master extends Logging {
   def main(args: Array[String]): Unit = {
     val conf = new CelebornConf()
     val masterArgs = new MasterArguments(args, conf)
-    val master = new Master(conf, masterArgs)
-    master.initialize()
+    try {
+      val master = new Master(conf, masterArgs)
+      master.initialize()
+    } catch {
+      case e: Throwable =>
+        logError("Initialize master failed.", e)
+        System.exit(-1)
+    }
   }
 }
