@@ -41,6 +41,7 @@ import org.apache.celeborn.common.protocol.PartitionSplitMode;
 import org.apache.celeborn.common.protocol.PartitionType;
 import org.apache.celeborn.common.protocol.StorageInfo;
 import org.apache.celeborn.common.unsafe.Platform;
+import org.apache.celeborn.common.util.FileChannelUtils;
 import org.apache.celeborn.service.deploy.worker.WorkerSource;
 import org.apache.celeborn.service.deploy.worker.congestcontrol.CongestionController;
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager;
@@ -81,6 +82,10 @@ public abstract class FileWriter implements DeviceObserver {
   protected boolean deleted = false;
   private RoaringBitmap mapIdBitMap = null;
   protected final FlushNotifier notifier = new FlushNotifier();
+  // It's only needed when graceful shutdown is enabled
+  private String shuffleKey;
+  private StorageManager storageManager;
+  private boolean workerGracefulShutdown;
 
   public FileWriter(
       FileInfo fileInfo,
@@ -97,6 +102,7 @@ public abstract class FileWriter implements DeviceObserver {
     this.flusher = flusher;
     this.flushWorkerIndex = flusher.getWorkerIndex();
     this.writerCloseTimeoutMs = conf.workerWriterCloseTimeoutMs();
+    this.workerGracefulShutdown = conf.workerGracefulShutdown();
     this.splitThreshold = splitThreshold;
     this.deviceMonitor = deviceMonitor;
     this.splitMode = splitMode;
@@ -244,6 +250,10 @@ public abstract class FileWriter implements DeviceObserver {
     void run() throws IOException;
   }
 
+  public boolean isClosed() {
+    return closed;
+  }
+
   protected synchronized long close(
       RunnableWithIOException tryClose,
       RunnableWithIOException streamClose,
@@ -273,9 +283,7 @@ public abstract class FileWriter implements DeviceObserver {
         if (channel != null) {
           channel.close();
         }
-        if (fileInfo.isHdfs()) {
-          streamClose.run();
-        }
+        streamClose.run();
       } catch (IOException e) {
         logger.warn("close file writer {} failed", this, e);
       }
@@ -288,6 +296,9 @@ public abstract class FileWriter implements DeviceObserver {
         deviceMonitor.unregisterFileWriter(this);
       }
     }
+    if (workerGracefulShutdown) {
+      storageManager.notifyFileInfoCommitted(shuffleKey, getFile().getName(), fileInfo);
+    }
     return fileInfo.getFileLength();
   }
 
@@ -440,4 +451,12 @@ public abstract class FileWriter implements DeviceObserver {
   public PartitionType getPartitionType() {
     return partitionType;
   }
+
+  public void setShuffleKey(String shuffleKey) {
+    this.shuffleKey = shuffleKey;
+  }
+
+  public void setStorageManager(StorageManager storageManager) {
+    this.storageManager = storageManager;
+  }
 }
