@@ -38,7 +38,7 @@ import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.rpc._
 import org.apache.celeborn.common.util.{JavaUtils, Utils}
-import org.apache.celeborn.service.deploy.worker.storage.StorageManager
+import org.apache.celeborn.service.deploy.worker.storage.{FileWriter, MapPartitionFileWriter, StorageManager}
 
 private[deploy] class Controller(
     override val rpcEnv: RpcEnv,
@@ -90,7 +90,8 @@ private[deploy] class Controller(
           partitionType,
           rangeReadFilter,
           userIdentifier,
-          pushDataTimeout) =>
+          pushDataTimeout,
+          partitionSplitEnabled) =>
       val shuffleKey = Utils.makeShuffleKey(applicationId, shuffleId)
       workerSource.sample(WorkerSource.RESERVE_SLOTS_TIME, shuffleKey) {
         logDebug(s"Received ReserveSlots request, $shuffleKey, " +
@@ -107,7 +108,8 @@ private[deploy] class Controller(
           partitionType,
           rangeReadFilter,
           userIdentifier,
-          pushDataTimeout)
+          pushDataTimeout,
+          partitionSplitEnabled)
         logDebug(s"ReserveSlots for $shuffleKey finished.")
       }
 
@@ -136,7 +138,8 @@ private[deploy] class Controller(
       partitionType: PartitionType,
       rangeReadFilter: Boolean,
       userIdentifier: UserIdentifier,
-      pushDataTimeout: Long): Unit = {
+      pushDataTimeout: Long,
+      partitionSplitEnabled: Boolean): Unit = {
     val shuffleKey = Utils.makeShuffleKey(applicationId, shuffleId)
     if (shutdown.get()) {
       val msg = "Current worker is shutting down!"
@@ -167,7 +170,8 @@ private[deploy] class Controller(
             splitMode,
             partitionType,
             rangeReadFilter,
-            userIdentifier)
+            userIdentifier,
+            partitionSplitEnabled)
           primaryLocs.add(new WorkingPartition(location, writer))
         } else {
           primaryLocs.add(location)
@@ -206,7 +210,8 @@ private[deploy] class Controller(
             splitMode,
             partitionType,
             rangeReadFilter,
-            userIdentifier)
+            userIdentifier,
+            partitionSplitEnabled)
           replicaLocs.add(new WorkingPartition(location, writer))
         } else {
           replicaLocs.add(location)
@@ -283,6 +288,7 @@ private[deploy] class Controller(
                 }
 
                 val fileWriter = location.asInstanceOf[WorkingPartition].getFileWriter
+                waitMapPartitionRegionFinished(fileWriter, conf.workerShuffleCommitTimeout)
                 val bytes = fileWriter.close()
                 if (bytes > 0L) {
                   if (fileWriter.getStorageInfo == null) {
@@ -321,6 +327,23 @@ private[deploy] class Controller(
     future
   }
 
+  private def waitMapPartitionRegionFinished(fileWriter: FileWriter, waitTimeout: Long): Unit = {
+    if (fileWriter.isInstanceOf[MapPartitionFileWriter]) {
+      val delta = 100
+      var times = 0
+      while (delta * times < waitTimeout) {
+        if (fileWriter.asInstanceOf[MapPartitionFileWriter].isRegionFinished) {
+          logDebug(s"CommitFile succeed to waitMapPartitionRegionFinished ${fileWriter.getFile.getAbsolutePath}")
+          return
+        }
+        Thread.sleep(delta)
+        times += 1
+      }
+      logWarning(
+        s"CommitFile faield to waitMapPartitionRegionFinished ${fileWriter.getFile.getAbsolutePath}")
+    }
+  }
+
   private def handleCommitFiles(
       context: RpcCallContext,
       shuffleKey: String,
