@@ -22,18 +22,20 @@ import java.util.concurrent.{ConcurrentHashMap, ThreadPoolExecutor}
 import java.util.concurrent.atomic.{AtomicBoolean, AtomicIntegerArray}
 
 import com.google.common.base.Throwables
+import com.google.protobuf.GeneratedMessageV3
 import io.netty.buffer.ByteBuf
 
 import org.apache.celeborn.common.exception.{AlreadyClosedException, CelebornIOException}
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{WorkerInfo, WorkerPartitionLocationInfo}
+import org.apache.celeborn.common.meta.{DiskStatus, WorkerInfo, WorkerPartitionLocationInfo}
 import org.apache.celeborn.common.metrics.source.Source
 import org.apache.celeborn.common.network.buffer.{NettyManagedBuffer, NioManagedBuffer}
 import org.apache.celeborn.common.network.client.{RpcResponseCallback, TransportClient, TransportClientFactory}
-import org.apache.celeborn.common.network.protocol.{Message, PushData, PushDataHandShake, PushMergedData, RegionFinish, RegionStart, RequestMessage, RpcFailure, RpcRequest, RpcResponse}
+import org.apache.celeborn.common.network.protocol.{Message, PushData, PushDataHandShake, PushMergedData, RegionFinish, RegionStart, RequestMessage, RpcFailure, RpcRequest, RpcResponse, TransportMessage}
 import org.apache.celeborn.common.network.protocol.Message.Type
 import org.apache.celeborn.common.network.server.BaseMessageHandler
-import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType}
+import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType, PbPushDataHandShake, PbRegionFinish, PbRegionStart}
+import org.apache.celeborn.common.protocol.PbPartitionLocation.Mode
 import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.unsafe.Platform
 import org.apache.celeborn.common.util.Utils
@@ -175,9 +177,9 @@ class PushDataHandler extends BaseMessageHandler with Logging {
     if (location == null) {
       val (mapId, attemptId) = getMapAttempt(body)
       // MapperAttempts for a shuffle exists after any CommitFiles request succeeds.
-      // A shuffle can trigger multiple CommitFiles requests, for reasons like: Hard-Split happens, StageEnd.
+      // A shuffle can trigger multiple CommitFiles requests, for reasons like: HARD_SPLIT happens, StageEnd.
       // If MapperAttempts but the value is -1 for the mapId(-1 means the map has not yet finished),
-      // it's probably because commitFiles for Had-Split happens.
+      // it's probably because commitFiles for HARD_SPLIT happens.
       if (shuffleMapperAttempts.containsKey(shuffleKey)) {
         if (-1 != shuffleMapperAttempts.get(shuffleKey).get(mapId)) {
           // partition data has already been committed
@@ -195,7 +197,7 @@ class PushDataHandler extends BaseMessageHandler with Logging {
         if (storageManager.shuffleKeySet().contains(shuffleKey)) {
           // If there is no shuffle key in shuffleMapperAttempts but there is shuffle key
           // in StorageManager. This partition should be HARD_SPLIT partition and
-          // after worker restart, some task still push data to this HARD_SPLIT partition.
+          // after worker restart, some tasks still push data to this HARD_SPLIT partition.
           logInfo(s"[Case2] Receive push data for committed hard split partition of " +
             s"(shuffle $shuffleKey, map $mapId attempt $attemptId)")
           callbackWithTimer.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
@@ -238,6 +240,14 @@ class PushDataHandler extends BaseMessageHandler with Logging {
 
     fileWriter.incrementPendingWrites()
 
+    if (fileWriter.isClosed) {
+      logWarning(
+        s"[handlePushData] FileWriter is already closed! File path ${fileWriter.getFileInfo.getFilePath}")
+      callbackWithTimer.onFailure(new CelebornIOException("File already closed!"))
+      fileWriter.decrementPendingWrites()
+      return;
+    }
+
     // for primary, send data to replica
     if (doReplicate) {
       pushData.body().retain()
@@ -376,8 +386,9 @@ class PushDataHandler extends BaseMessageHandler with Logging {
             shuffleMapperAttempts.get(shuffleKey).get(mapId)
           } else -1
         // TODO just info log for ended attempt
-        logWarning(s"Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
-          s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
+        logError(
+          s"[handlePushData] Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
+            s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
       case e: Exception =>
         logError("Exception encountered when write.", e)
     }
@@ -437,9 +448,9 @@ class PushDataHandler extends BaseMessageHandler with Logging {
       if (loc == null) {
         val (mapId, attemptId) = getMapAttempt(body)
         // MapperAttempts for a shuffle exists after any CommitFiles request succeeds.
-        // A shuffle can trigger multiple CommitFiles requests, for reasons like: Hard-Split happens, StageEnd.
+        // A shuffle can trigger multiple CommitFiles requests, for reasons like: HARD_SPLIT happens, StageEnd.
         // If MapperAttempts but the value is -1 for the mapId(-1 means the map has not yet finished),
-        // it's probably because commitFiles for Had-Split happens.
+        // it's probably because commitFiles for HARD_SPLIT happens.
         if (shuffleMapperAttempts.containsKey(shuffleKey)) {
           if (-1 != shuffleMapperAttempts.get(shuffleKey).get(mapId)) {
             logInfo(s"Receive push merged data from speculative " +
@@ -448,7 +459,7 @@ class PushDataHandler extends BaseMessageHandler with Logging {
             callbackWithTimer.onSuccess(
               ByteBuffer.wrap(Array[Byte](StatusCode.STAGE_ENDED.getValue)))
           } else {
-            logInfo(s"Receive push merged data for committed hard split partition of " +
+            logInfo(s"[Case1] Receive push merged data for committed hard split partition of " +
               s"(shuffle $shuffleKey, map $mapId attempt $attemptId)")
             callbackWithTimer.onSuccess(
               ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
@@ -457,8 +468,8 @@ class PushDataHandler extends BaseMessageHandler with Logging {
           if (storageManager.shuffleKeySet().contains(shuffleKey)) {
             // If there is no shuffle key in shuffleMapperAttempts but there is shuffle key
             // in StorageManager. This partition should be HARD_SPLIT partition and
-            // after worker restart, some task still push data to this HARD_SPLIT partition.
-            logInfo(s"Receive push merged data for committed hard split partition of " +
+            // after worker restart, some tasks still push data to this HARD_SPLIT partition.
+            logInfo(s"[Case2] Receive push merged data for committed hard split partition of " +
               s"(shuffle $shuffleKey, map $mapId attempt $attemptId)")
             callbackWithTimer.onSuccess(
               ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
@@ -499,6 +510,15 @@ class PushDataHandler extends BaseMessageHandler with Logging {
     }
     fileWriters.foreach(_.incrementPendingWrites())
 
+    val closedFileWriter = fileWriters.find(_.isClosed)
+    if (closedFileWriter.isDefined) {
+      logWarning(
+        s"[handlePushMergedData] FileWriter is already closed! File path ${closedFileWriter.get.getFileInfo.getFilePath}")
+      callbackWithTimer.onFailure(new CelebornIOException("File already closed!"))
+      fileWriters.foreach(_.decrementPendingWrites())
+      return
+    }
+
     // for primary, send data to replica
     if (doReplicate) {
       pushMergedData.body().retain()
@@ -649,8 +669,9 @@ class PushDataHandler extends BaseMessageHandler with Logging {
               shuffleMapperAttempts.get(shuffleKey).get(mapId)
             } else -1
           // TODO just info log for ended attempt
-          logWarning(s"Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
-            s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
+          logError(
+            s"[handlePushMergedData] Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
+              s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
         case e: Exception =>
           logError("Exception encountered when write.", e)
       }
@@ -779,18 +800,8 @@ class PushDataHandler extends BaseMessageHandler with Logging {
         pushData.`type`(),
         shuffleKey,
         pushData.partitionUniqueId,
-        null,
         location,
-        callback,
-        wrappedCallback)) return
-
-    // During worker shutdown, worker will return HARD_SPLIT for all existed partition.
-    // This should before return exception to make current push request revive and retry.
-    if (shutdown.get()) {
-      logInfo(s"Push data return HARD_SPLIT for shuffle $shuffleKey since worker shutdown.")
-      callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
-      return
-    }
+        callback)) return
 
     val fileWriter =
       getFileWriterAndCheck(pushData.`type`(), location, isPrimary, callback) match {
@@ -802,6 +813,14 @@ class PushDataHandler extends BaseMessageHandler with Logging {
 
     fileWriter.incrementPendingWrites()
 
+    if (fileWriter.isClosed) {
+      logWarning(
+        s"[handleMapPartitionPushData] FileWriter is already closed! File path ${fileWriter.getFileInfo.getFilePath}")
+      callback.onFailure(new CelebornIOException("File already closed!"))
+      fileWriter.decrementPendingWrites()
+      return;
+    }
+
     // for primary, send data to replica
     if (location.hasPeer && isPrimary) {
       // to do
@@ -821,53 +840,129 @@ class PushDataHandler extends BaseMessageHandler with Logging {
             shuffleMapperAttempts.get(shuffleKey).get(mapId)
           } else -1
         // TODO just info log for ended attempt
-        logWarning(s"Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
-          s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
+        logError(
+          s"[handleMapPartitionPushData] Append data failed for task(shuffle $shuffleKey, map $mapId, attempt" +
+            s" $attemptId), caused by AlreadyClosedException, endedAttempt $endedAttempt, error message: ${e.getMessage}")
       case e: Exception =>
         logError("Exception encountered when write.", e)
     }
   }
 
   private def handleRpcRequest(client: TransportClient, rpcRequest: RpcRequest): Unit = {
-    val msg = Message.decode(rpcRequest.body().nioByteBuffer())
     val requestId = rpcRequest.requestId
-    val (mode, shuffleKey, partitionUniqueId, checkSplit) = msg match {
-      case p: PushDataHandShake => (p.mode, p.shuffleKey, p.partitionUniqueId, false)
-      case rs: RegionStart => (rs.mode, rs.shuffleKey, rs.partitionUniqueId, true)
-      case rf: RegionFinish => (rf.mode, rf.shuffleKey, rf.partitionUniqueId, false)
-    }
+    val (pbMsg, msg, isLegacy, messageType, mode, shuffleKey, partitionUniqueId, checkSplit) =
+      mapPartitionRpcRequest(rpcRequest)
     handleCore(
       client,
       rpcRequest,
       requestId,
       () =>
-        handleRpcRequestCore(
-          mode,
+        handleMapPartitionRpcRequestCore(
+          requestId,
+          pbMsg,
           msg,
+          isLegacy,
+          messageType,
+          mode,
           shuffleKey,
           partitionUniqueId,
-          requestId,
           checkSplit,
           new SimpleRpcResponseCallback(
             client,
             requestId,
             shuffleKey)))
-
   }
 
-  private def handleRpcRequestCore(
-      mode: Byte,
-      message: Message,
+  private def mapPartitionRpcRequest(rpcRequest: RpcRequest)
+      : Tuple8[GeneratedMessageV3, Message, Boolean, Type, Mode, String, String, Boolean] = {
+    try {
+      val msg = TransportMessage.fromByteBuffer(
+        rpcRequest.body().nioByteBuffer()).getParsedPayload.asInstanceOf[GeneratedMessageV3]
+      msg match {
+        case p: PbPushDataHandShake =>
+          (
+            msg,
+            null,
+            false,
+            Type.PUSH_DATA_HAND_SHAKE,
+            p.getMode,
+            p.getShuffleKey,
+            p.getPartitionUniqueId,
+            true)
+        case rs: PbRegionStart =>
+          (
+            msg,
+            null,
+            false,
+            Type.REGION_START,
+            rs.getMode,
+            rs.getShuffleKey,
+            rs.getPartitionUniqueId,
+            true)
+        case rf: PbRegionFinish =>
+          (
+            msg,
+            null,
+            false,
+            Type.REGION_FINISH,
+            rf.getMode,
+            rf.getShuffleKey,
+            rf.getPartitionUniqueId,
+            false)
+      }
+    } catch {
+      case _: Exception =>
+        val msg = Message.decode(rpcRequest.body().nioByteBuffer())
+        msg match {
+          case p: PushDataHandShake =>
+            (
+              null,
+              msg,
+              true,
+              Type.PUSH_DATA_HAND_SHAKE,
+              Mode.forNumber(p.mode),
+              p.shuffleKey,
+              p.partitionUniqueId,
+              true)
+          case rs: RegionStart =>
+            (
+              null,
+              msg,
+              true,
+              Type.REGION_START,
+              Mode.forNumber(rs.mode),
+              rs.shuffleKey,
+              rs.partitionUniqueId,
+              true)
+          case rf: RegionFinish =>
+            (
+              null,
+              msg,
+              true,
+              Type.REGION_FINISH,
+              Mode.forNumber(rf.mode),
+              rf.shuffleKey,
+              rf.partitionUniqueId,
+              false)
+        }
+    }
+  }
+
+  private def handleMapPartitionRpcRequestCore(
+      requestId: Long,
+      pbMsg: GeneratedMessageV3,
+      msg: Message,
+      isLegacy: Boolean,
+      messageType: Message.Type,
+      mode: Mode,
       shuffleKey: String,
       partitionUniqueId: String,
-      requestId: Long,
       checkSplit: Boolean,
       callback: RpcResponseCallback): Unit = {
-    val isPrimary = PartitionLocation.getMode(mode) == PartitionLocation.Mode.PRIMARY
-    val messageType = message.`type`()
     log.debug(
       s"requestId:$requestId, pushdata rpc:$messageType, mode:$mode, shuffleKey:$shuffleKey, " +
         s"partitionUniqueId:$partitionUniqueId")
+    val isPrimary = mode == Mode.Primary
     val (workerSourcePrimary, workerSourceReplica) =
       messageType match {
         case Type.PUSH_DATA_HAND_SHAKE =>
@@ -904,10 +999,8 @@ class PushDataHandler extends BaseMessageHandler with Logging {
         messageType,
         shuffleKey,
         partitionUniqueId,
-        null,
         location,
-        callback,
-        wrappedCallback)) return
+        callback)) return
 
     val fileWriter =
       getFileWriterAndCheck(messageType, location, isPrimary, callback) match {
@@ -915,18 +1008,53 @@ class PushDataHandler extends BaseMessageHandler with Logging {
         case (false, f: FileWriter) => f
       }
 
-    if (checkSplit && checkDiskFullAndSplit(fileWriter, isPrimary, null, callback)) return
+    // During worker shutdown, worker will return HARD_SPLIT for all existed partition.
+    // This should before return exception to make current push request revive and retry.
+    val isPartitionSplitEnabled = fileWriter.asInstanceOf[
+      MapPartitionFileWriter].getFileInfo.isPartitionSplitEnabled
+
+    if (shutdown.get() && (messageType == Type.REGION_START || messageType ==
+        Type.PUSH_DATA_HAND_SHAKE) && isPartitionSplitEnabled) {
+      logInfo(s"$messageType return HARD_SPLIT for shuffle $shuffleKey since worker shutdown.")
+      callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
+      return
+    }
+
+    if (checkSplit && (messageType == Type.REGION_START || messageType ==
+        Type.PUSH_DATA_HAND_SHAKE) && isPartitionSplitEnabled && checkDiskFullAndSplit(
+        fileWriter,
+        isPrimary,
+        null,
+        callback)) return
 
     try {
       messageType match {
         case Type.PUSH_DATA_HAND_SHAKE =>
+          val (numPartitions, bufferSize) =
+            if (isLegacy)
+              (
+                msg.asInstanceOf[PushDataHandShake].numPartitions,
+                msg.asInstanceOf[PushDataHandShake].bufferSize)
+            else
+              (
+                pbMsg.asInstanceOf[PbPushDataHandShake].getNumPartitions,
+                pbMsg.asInstanceOf[PbPushDataHandShake].getBufferSize)
           fileWriter.asInstanceOf[MapPartitionFileWriter].pushDataHandShake(
-            message.asInstanceOf[PushDataHandShake].numPartitions,
-            message.asInstanceOf[PushDataHandShake].bufferSize)
+            numPartitions,
+            bufferSize)
         case Type.REGION_START =>
+          val (currentRegionIndex, isBroadcast) =
+            if (isLegacy)
+              (
+                msg.asInstanceOf[RegionStart].currentRegionIndex,
+                msg.asInstanceOf[RegionStart].isBroadcast)
+            else
+              (
+                pbMsg.asInstanceOf[PbRegionStart].getCurrentRegionIndex,
+                Boolean.box(pbMsg.asInstanceOf[PbRegionStart].getIsBroadcast))
           fileWriter.asInstanceOf[MapPartitionFileWriter].regionStart(
-            message.asInstanceOf[RegionStart].currentRegionIndex,
-            message.asInstanceOf[RegionStart].isBroadcast)
+            currentRegionIndex,
+            isBroadcast)
         case Type.REGION_FINISH =>
           fileWriter.asInstanceOf[MapPartitionFileWriter].regionFinish()
         case _ => throw new IllegalArgumentException(s"Not support $messageType yet")
@@ -1002,10 +1130,8 @@ class PushDataHandler extends BaseMessageHandler with Logging {
       messageType: Message.Type,
       shuffleKey: String,
       partitionUniqueId: String,
-      body: ByteBuf,
       location: PartitionLocation,
-      callback: RpcResponseCallback,
-      wrappedCallback: RpcResponseCallback): Boolean = {
+      callback: RpcResponseCallback): Boolean = {
     if (location == null) {
       val msg =
         s"Partition location wasn't found for task(shuffle $shuffleKey, uniqueId $partitionUniqueId)."
@@ -1068,9 +1194,12 @@ class PushDataHandler extends BaseMessageHandler with Logging {
     if (fileWriter.flusher.isInstanceOf[HdfsFlusher]) {
       return false
     }
-    val diskFull = workerInfo.diskInfos
+    val diskInfo = workerInfo.diskInfos
       .get(fileWriter.flusher.asInstanceOf[LocalFlusher].mountPoint)
-      .actualUsableSpace < diskReserveSize
+
+    val diskFull = diskInfo.status.equals(
+      DiskStatus.HIGH_DISK_USAGE) || diskInfo.actualUsableSpace < diskReserveSize
+
     diskFull
   }
 
@@ -1080,6 +1209,15 @@ class PushDataHandler extends BaseMessageHandler with Logging {
       softSplit: AtomicBoolean,
       callback: RpcResponseCallback): Boolean = {
     val diskFull = checkDiskFull(fileWriter)
+    logDebug(
+      s"""
+         |CheckDiskFullAndSplit in
+         |diskFull:$diskFull,
+         |partitionSplitMinimumSize: $partitionSplitMinimumSize,
+         |splitThreshold:${fileWriter.getSplitThreshold()},
+         |fileLength:${fileWriter.getFileInfo.getFileLength}
+         |fileName:${fileWriter.getFileInfo.getFilePath}
+         |""".stripMargin)
     if (workerPartitionSplitEnabled && ((diskFull && fileWriter.getFileInfo.getFileLength > partitionSplitMinimumSize) ||
         (isPrimary && fileWriter.getFileInfo.getFileLength > fileWriter.getSplitThreshold()))) {
       if (softSplit != null && fileWriter.getSplitMode == PartitionSplitMode.SOFT &&
@@ -1087,6 +1225,15 @@ class PushDataHandler extends BaseMessageHandler with Logging {
         softSplit.set(true)
       } else {
         callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
+        logDebug(
+          s"""
+             |CheckDiskFullAndSplit hardSplit
+             |diskFull:$diskFull,
+             |partitionSplitMinimumSize:$partitionSplitMinimumSize,
+             |splitThreshold:${fileWriter.getSplitThreshold()},
+             |fileLength:${fileWriter.getFileInfo.getFileLength},
+             |fileName:${fileWriter.getFileInfo.getFilePath}
+             |""".stripMargin)
         return true
       }
     }
@@ -1100,6 +1247,23 @@ class PushDataHandler extends BaseMessageHandler with Logging {
       pushClientFactory.createClient(host, port, partitionId)
     }
   }
+
+  /**
+   * Invoked when the channel associated with the given client is active.
+   */
+  override def channelActive(client: TransportClient): Unit = {
+    workerSource.incCounter(WorkerSource.ACTIVE_CONNECTION_COUNT)
+    super.channelActive(client)
+  }
+
+  /**
+   * Invoked when the channel associated with the given client is inactive.
+   * No further requests will come from this client.
+   */
+  override def channelInactive(client: TransportClient): Unit = {
+    workerSource.incCounter(WorkerSource.ACTIVE_CONNECTION_COUNT, -1)
+    super.channelInactive(client)
+  }
 }
 
 object PushDataHandler {
