@@ -42,12 +42,14 @@ import org.apache.celeborn.common.protocol.{PartitionType, PbRegisterWorkerRespo
 import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.quota.ResourceConsumption
 import org.apache.celeborn.common.rpc._
-import org.apache.celeborn.common.util.{JavaUtils, ShutdownHookManager, ThreadUtils, Utils}
+import org.apache.celeborn.common.util.{CelebornExitKind, JavaUtils, ShutdownHookManager, ThreadUtils, Utils}
 // Can Remove this if celeborn don't support scala211 in future
 import org.apache.celeborn.common.util.FunctionConverter._
 import org.apache.celeborn.server.common.{HttpService, Service}
+import org.apache.celeborn.service.deploy.worker.WorkerSource.ACTIVE_CONNECTION_COUNT
 import org.apache.celeborn.service.deploy.worker.congestcontrol.CongestionController
 import org.apache.celeborn.service.deploy.worker.memory.{ChannelsLimiter, MemoryManager}
+import org.apache.celeborn.service.deploy.worker.memory.MemoryManager.ServingState
 import org.apache.celeborn.service.deploy.worker.storage.{PartitionFilesSorter, StorageManager}
 
 private[celeborn] class Worker(
@@ -77,12 +79,18 @@ private[celeborn] class Worker(
   private val WORKER_SHUTDOWN_PRIORITY = 100
   val shutdown = new AtomicBoolean(false)
   private val gracefulShutdown = conf.workerGracefulShutdown
-  assert(
-    !gracefulShutdown || (gracefulShutdown &&
-      conf.workerRpcPort > 0 && conf.workerFetchPort > 0 &&
-      conf.workerPushPort > 0 && conf.workerReplicatePort > 0),
-    "If enable graceful shutdown, the worker should use stable server port.")
+  private var exitKind = CelebornExitKind.EXIT_IMMEDIATELY
   if (gracefulShutdown) {
+    val checkPortMap = Map(
+      WORKER_RPC_PORT -> conf.workerRpcPort,
+      WORKER_FETCH_PORT -> conf.workerFetchPort,
+      WORKER_PUSH_PORT -> conf.workerPushPort,
+      WORKER_REPLICATE_PORT -> conf.workerReplicatePort)
+    assert(
+      !checkPortMap.values.exists(_ == 0),
+      "If enable graceful shutdown, the worker should use non-zero port. " +
+        s"${checkPortMap.map { case (k, v) => k.key + "=" + v }.mkString(", ")}")
+    exitKind = CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN
     try {
       val recoverRoot = new File(conf.workerGracefulShutdownRecoverPath)
       if (!recoverRoot.exists()) {
@@ -293,6 +301,22 @@ private[celeborn] class Worker(
   workerSource.addGauge(WorkerSource.READ_BUFFER_ALLOCATED_COUNT) { () =>
     memoryManager.getAllocatedReadBuffers
   }
+  workerSource.addGauge(WorkerSource.ACTIVE_SHUFFLE_SIZE) { () =>
+    storageManager.getActiveShuffleSize()
+  }
+  workerSource.addGauge(WorkerSource.PAUSE_PUSH_DATA_TIME) { () =>
+    memoryManager.getPausePushDataTime
+  }
+
+  private def highWorkload: Boolean = {
+    (memoryManager.currentServingState, conf.workerActiveConnectionMax) match {
+      case (ServingState.PUSH_AND_REPLICATE_PAUSED, _) => true
+      case (ServingState.PUSH_PAUSED, _) => true
+      case (_, Some(activeConnectionMax)) =>
+        workerSource.getCounterCount(ACTIVE_CONNECTION_COUNT) >= activeConnectionMax
+      case _ => false
+    }
+  }
 
   private def heartbeatToMaster(): Unit = {
     val activeShuffleKeys = new JHashSet[String]()
@@ -320,7 +344,8 @@ private[celeborn] class Worker(
         diskInfos,
         resourceConsumption,
         activeShuffleKeys,
-        estimatedAppDiskUsage),
+        estimatedAppDiskUsage,
+        highWorkload),
       classOf[HeartbeatFromWorkerResponse])
     response.expiredShuffleKeys.asScala.foreach(shuffleKey => workerInfo.releaseSlots(shuffleKey))
     cleanTaskQueue.put(response.expiredShuffleKeys)
@@ -392,16 +417,12 @@ private[celeborn] class Worker(
     rpcEnv.awaitTermination()
   }
 
-  override def close(): Unit = synchronized {
-    shutdown(gracefulShutdown)
-  }
-
-  override def shutdown(graceful: Boolean): Unit = {
+  override def stop(exitKind: Int): Unit = {
     if (!stopped) {
       logInfo("Stopping Worker.")
 
       if (sendHeartbeatTask != null) {
-        if (graceful) {
+        if (exitKind == CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN) {
           sendHeartbeatTask.cancel(false)
         } else {
           sendHeartbeatTask.cancel(true)
@@ -409,45 +430,38 @@ private[celeborn] class Worker(
         sendHeartbeatTask = null
       }
       if (checkFastFailTask != null) {
-        if (graceful) {
+        if (exitKind == CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN) {
           checkFastFailTask.cancel(false)
         } else {
           checkFastFailTask.cancel(true)
         }
         checkFastFailTask = null
       }
-      if (graceful) {
+      if (exitKind == CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN) {
         forwardMessageScheduler.shutdown()
         replicateThreadPool.shutdown()
         commitThreadPool.shutdown()
         asyncReplyPool.shutdown()
-        partitionsSorter.close()
       } else {
         forwardMessageScheduler.shutdownNow()
         replicateThreadPool.shutdownNow()
         commitThreadPool.shutdownNow()
         asyncReplyPool.shutdownNow()
-        partitionsSorter.close()
-      }
-
-      if (null != storageManager) {
-        storageManager.close()
       }
+      partitionsSorter.close(exitKind)
+      storageManager.close(exitKind)
       memoryManager.close()
 
       masterClient.close()
-      replicateServer.shutdown(graceful)
-      fetchServer.shutdown(graceful)
-      pushServer.shutdown(graceful)
+      replicateServer.shutdown(exitKind)
+      fetchServer.shutdown(exitKind)
+      pushServer.shutdown(exitKind)
 
-      super.shutdown(graceful)
+      super.stop(exitKind)
 
       logInfo("Worker is stopped.")
       stopped = true
     }
-    if (!graceful) {
-      shutdown.set(true)
-    }
   }
 
   private def registerWithMaster(): Unit = {
@@ -570,55 +584,136 @@ private[celeborn] class Worker(
     sb.toString()
   }
 
+  override def exit(exitType: String): String = {
+    exitType match {
+      case "DECOMMISSION" =>
+        exitKind = CelebornExitKind.WORKER_DECOMMISSION
+        ShutdownHookManager.get().updateTimeout(
+          conf.workerDecommissionForceExitTimeout,
+          TimeUnit.MILLISECONDS)
+      case "GRACEFUL" =>
+        exitKind = CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN
+      case "IMMEDIATELY" =>
+        exitKind = CelebornExitKind.EXIT_IMMEDIATELY
+      case _ => // Use origin code
+    }
+    // Use the original EXIT_CODE
+    new Thread() {
+      override def run(): Unit = {
+        Thread.sleep(10000)
+        System.exit(0)
+      }
+    }.start()
+    val sb = new StringBuilder
+    sb.append("============================ Exit Worker =============================\n")
+    sb.append(s"Exit worker by $exitType triggered: \n")
+    sb.append(workerInfo.toString()).append("\n")
+    sb.toString()
+  }
+
+  def shutdownGracefully(): Unit = {
+    // During shutdown, to avoid allocate slots in this worker,
+    // add this worker to master's excluded list. When restart, register worker will
+    // make master remove this worker from excluded list.
+    try {
+      masterClient.askSync(
+        ReportWorkerUnavailable(List(workerInfo).asJava),
+        OneWayMessageResponse.getClass)
+    } catch {
+      case e: Throwable =>
+        logError(
+          s"Fail report to master, need wait PartitionLocation auto release: \n$partitionLocationInfo",
+          e)
+    }
+    shutdown.set(true)
+    val interval = conf.workerGracefulShutdownCheckSlotsFinishedInterval
+    val timeout = conf.workerGracefulShutdownCheckSlotsFinishedTimeoutMs
+    var waitTimes = 0
+
+    def waitTime: Long = waitTimes * interval
+
+    while (!partitionLocationInfo.isEmpty && waitTime < timeout) {
+      Thread.sleep(interval)
+      waitTimes += 1
+    }
+    if (partitionLocationInfo.isEmpty) {
+      logInfo(s"Waiting for all PartitionLocation released cost ${waitTime}ms.")
+    } else {
+      logWarning(s"Waiting for all PartitionLocation release cost ${waitTime}ms, " +
+        s"unreleased PartitionLocation: \n$partitionLocationInfo")
+    }
+  }
+
+  def decommissionWorker(): Unit = {
+    try {
+      masterClient.askSync(
+        ReportWorkerUnavailable(List(workerInfo).asJava),
+        OneWayMessageResponse.getClass)
+    } catch {
+      case e: Throwable =>
+        logError(
+          s"Fail report to master, need wait registered shuffle expired: " +
+            s"\n${storageManager.shuffleKeySet().asScala.mkString("[", ", ", "]")}",
+          e)
+    }
+    shutdown.set(true)
+    val interval = conf.workerDecommissionCheckInterval
+    val timeout = conf.workerDecommissionForceExitTimeout
+    var waitTimes = 0
+
+    def waitTime: Long = waitTimes * interval
+
+    while (!storageManager.shuffleKeySet().isEmpty && waitTime < timeout) {
+      Thread.sleep(interval)
+      waitTimes += 1
+    }
+    if (storageManager.shuffleKeySet().isEmpty) {
+      logInfo(s"Waiting for all shuffle expired cost ${waitTime}ms.")
+    } else {
+      logWarning(s"Waiting for all shuffle expired cost ${waitTime}ms, " +
+        s"unreleased shuffle: \n${storageManager.shuffleKeySet().asScala.mkString("[", ", ", "]")}")
+    }
+  }
+
+  def exitImmediately(): Unit = {
+    // During shutdown, to avoid allocate slots in this worker,
+    // add this worker to master's excluded list. When restart, register worker will
+    // make master remove this worker from excluded list.
+    try {
+      masterClient.askSync[PbWorkerLostResponse](
+        WorkerLost(
+          host,
+          rpcPort,
+          pushPort,
+          fetchPort,
+          replicatePort,
+          MasterClient.genRequestId()),
+        classOf[PbWorkerLostResponse])
+    } catch {
+      case e: Throwable =>
+        logError(
+          s"Fail report to master, need wait PartitionLocation auto release: \n$partitionLocationInfo",
+          e)
+    }
+    shutdown.set(true)
+  }
+
   ShutdownHookManager.get().addShutdownHook(
     new Thread(new Runnable {
       override def run(): Unit = {
         logInfo("Shutdown hook called.")
-        // During shutdown, to avoid allocate slots in this worker,
-        // add this worker to master's excluded list. When restart, register worker will
-        // make master remove this worker from excluded list.
-        try {
-          if (gracefulShutdown) {
-            masterClient.askSync(
-              ReportWorkerUnavailable(List(workerInfo).asJava),
-              OneWayMessageResponse.getClass)
-          } else {
-            masterClient.askSync[PbWorkerLostResponse](
-              WorkerLost(
-                host,
-                rpcPort,
-                pushPort,
-                fetchPort,
-                replicatePort,
-                MasterClient.genRequestId()),
-              classOf[PbWorkerLostResponse])
-          }
-        } catch {
-          case e: Throwable =>
-            logError(
-              s"Fail report to master, need wait PartitionLocation auto release: \n$partitionLocationInfo",
-              e)
+        exitKind match {
+          case CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN =>
+            logInfo("Worker start to shutdown gracefully")
+            shutdownGracefully()
+          case CelebornExitKind.WORKER_DECOMMISSION =>
+            logInfo("Worker start to decommission")
+            decommissionWorker()
+          case _ =>
+            logInfo("Worker start to exit immediately")
+            exitImmediately()
         }
-        shutdown.set(true)
-        if (gracefulShutdown) {
-          val interval = conf.workerGracefulShutdownCheckSlotsFinishedInterval
-          val timeout = conf.workerGracefulShutdownCheckSlotsFinishedTimeoutMs
-          var waitTimes = 0
-
-          def waitTime: Long = waitTimes * interval
-
-          while (!partitionLocationInfo.isEmpty && waitTime < timeout) {
-            Thread.sleep(interval)
-            waitTimes += 1
-          }
-          if (partitionLocationInfo.isEmpty) {
-            logInfo(s"Waiting for all PartitionLocation released cost ${waitTime}ms.")
-          } else {
-            logWarning(s"Waiting for all PartitionLocation release cost ${waitTime}ms, " +
-              s"unreleased PartitionLocation: \n$partitionLocationInfo")
-          }
-        }
-        close()
+        stop(exitKind)
       }
     }),
     WORKER_SHUTDOWN_PRIORITY)
@@ -639,8 +734,8 @@ private[deploy] object Worker extends Logging {
       conf.set(MASTER_ENDPOINTS.key, RpcAddress.fromCelebornURL(master).hostPort)
     }
 
-    val worker = new Worker(conf, workerArgs)
     try {
+      val worker = new Worker(conf, workerArgs)
       worker.initialize()
     } catch {
       case e: Throwable =>
