@@ -36,6 +36,7 @@ class WorkerSource(conf: CelebornConf) extends AbstractSource(conf, MetricsSyste
   addCounter(PUSH_DATA_HANDSHAKE_FAIL_COUNT)
   addCounter(REGION_START_FAIL_COUNT)
   addCounter(REGION_FINISH_FAIL_COUNT)
+  addCounter(ACTIVE_CONNECTION_COUNT)
 
   // add Timers
   addTimer(COMMIT_FILES_TIME)
@@ -56,6 +57,10 @@ class WorkerSource(conf: CelebornConf) extends AbstractSource(conf, MetricsSyste
   addTimer(TAKE_BUFFER_TIME)
   addTimer(SORT_TIME)
 
+  def getCounterCount(metricsName: String): Long = {
+    val metricNameWithLabel = metricNameWithCustomizedLabels(metricsName, Map.empty)
+    namedCounters.get(metricNameWithLabel).counter.getCount
+  }
   // start cleaner thread
   startCleaner()
 }
@@ -85,6 +90,7 @@ object WorkerSource {
   val REPLICA_REGION_START_TIME = "ReplicaRegionStartTime"
   val PRIMARY_REGION_FINISH_TIME = "PrimaryRegionFinishTime"
   val REPLICA_REGION_FINISH_TIME = "ReplicaRegionFinishTime"
+  val PAUSE_PUSH_DATA_TIME = "PausePushDataTime"
 
   // flush
   val TAKE_BUFFER_TIME = "TakeBufferTime"
@@ -94,6 +100,8 @@ object WorkerSource {
   // slots
   val SLOTS_ALLOCATED = "SlotsAllocated"
 
+  val ACTIVE_CONNECTION_COUNT = "ActiveConnectionCount"
+
   // memory
   val NETTY_MEMORY = "NettyMemory"
   val SORT_TIME = "SortTime"
@@ -120,4 +128,7 @@ object WorkerSource {
   val POTENTIAL_CONSUME_SPEED = "PotentialConsumeSpeed"
   val USER_PRODUCE_SPEED = "UserProduceSpeed"
   val WORKER_CONSUME_SPEED = "WorkerConsumeSpeed"
+
+  // active shuffle size
+  val ACTIVE_SHUFFLE_SIZE = "ActiveShuffleSize"
 }
