@@ -21,7 +21,7 @@ import java.io.{File, IOException}
 import java.nio.charset.StandardCharsets
 import java.nio.file.{FileAlreadyExistsException, Files, Paths}
 import java.util
-import java.util.concurrent.{ConcurrentHashMap, ThreadPoolExecutor, TimeUnit}
+import java.util.concurrent.{ConcurrentHashMap, ScheduledExecutorService, ThreadPoolExecutor, TimeUnit}
 import java.util.concurrent.atomic.AtomicInteger
 import java.util.function.{BiConsumer, IntUnaryOperator}
 
@@ -31,7 +31,7 @@ import scala.concurrent.duration._
 import io.netty.buffer.PooledByteBufAllocator
 import org.apache.hadoop.fs.{FileSystem, Path}
 import org.apache.hadoop.fs.permission.FsPermission
-import org.iq80.leveldb.DB
+import org.iq80.leveldb.{DB, WriteOptions}
 
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.exception.CelebornException
@@ -42,7 +42,7 @@ import org.apache.celeborn.common.metrics.source.AbstractSource
 import org.apache.celeborn.common.network.util.{NettyUtils, TransportConf}
 import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType}
 import org.apache.celeborn.common.quota.ResourceConsumption
-import org.apache.celeborn.common.util.{CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
+import org.apache.celeborn.common.util.{CelebornExitKind, CelebornHadoopUtils, JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 import org.apache.celeborn.service.deploy.worker._
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager.MemoryPressureListener
 import org.apache.celeborn.service.deploy.worker.storage.StorageManager.hadoopFs
@@ -132,7 +132,13 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   val (hdfsFlusher, _totalHdfsFlusherThread) =
     if (hasHDFSStorage) {
       logInfo(s"Initialize HDFS support with path ${hdfsDir}")
-      StorageManager.hadoopFs = CelebornHadoopUtils.getHadoopFS(conf)
+      try {
+        StorageManager.hadoopFs = CelebornHadoopUtils.getHadoopFS(conf)
+      } catch {
+        case e: Exception =>
+          logError("Celeborn initialize HDFS failed.", e)
+          throw e
+      }
       (
         Some(new HdfsFlusher(
           workerSource,
@@ -179,9 +185,16 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     JavaUtils.newConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]]()
   private val RECOVERY_FILE_NAME = "recovery.ldb"
   private var db: DB = null
+  private var saveCommittedFileInfosExecutor: ScheduledExecutorService = _
+  private val saveCommittedFileInfoBySyncMethod =
+    conf.workerGracefulShutdownSaveCommittedFileInfoSync
+  private val saveCommittedFileInfoInterval =
+    conf.workerGracefulShutdownSaveCommittedFileInfoInterval
+  private var committedFileInfos: ConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]] = _
   // ShuffleClient can fetch data from a restarted worker only
   // when the worker's fetching port is stable.
-  if (conf.workerGracefulShutdown) {
+  val workerGracefulShutdown = conf.workerGracefulShutdown
+  if (workerGracefulShutdown) {
     try {
       val recoverFile = new File(conf.workerGracefulShutdownRecoverPath, RECOVERY_FILE_NAME)
       this.db = LevelDBProvider.initLevelDB(recoverFile, CURRENT_VERSION)
@@ -191,6 +204,28 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         logError("Init level DB failed:", e)
         this.db = null
     }
+    committedFileInfos =
+      JavaUtils.newConcurrentHashMap[String, ConcurrentHashMap[String, FileInfo]]()
+    saveCommittedFileInfosExecutor =
+      ThreadUtils.newDaemonSingleThreadScheduledExecutor(
+        "StorageManager-save-committed-fileinfo-thread")
+    saveCommittedFileInfosExecutor.scheduleAtFixedRate(
+      new Runnable {
+        override def run(): Unit = {
+          if (!committedFileInfos.isEmpty) {
+            logInfo(s"Save committed fileinfo with ${committedFileInfos.size()} shuffle keys")
+            committedFileInfos.asScala.foreach { case (shuffleKey, files) =>
+              db.put(
+                dbShuffleKey(shuffleKey),
+                PbSerDeUtils.toPbFileInfoMap(files),
+                new WriteOptions().sync(saveCommittedFileInfoBySyncMethod))
+            }
+          }
+        }
+      },
+      saveCommittedFileInfoInterval,
+      saveCommittedFileInfoInterval,
+      TimeUnit.MILLISECONDS)
   }
   cleanupExpiredAppDirs()
   if (!checkIfWorkingDirCleaned) {
@@ -228,10 +263,18 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     }
   }
 
-  def updateFileInfosInDB(): Unit = {
-    fileInfos.asScala.foreach { case (shuffleKey, files) =>
+  def saveAllCommittedFileInfosToDB(): Unit = {
+    // save committed fileinfo to DB should be done within the time of saveCommittedFileInfoInterval
+    saveCommittedFileInfosExecutor.awaitTermination(saveCommittedFileInfoInterval, MILLISECONDS)
+    // graceful shutdown might be timed out, persist all committed fileinfos to levelDB
+    // final flush write through
+    committedFileInfos.asScala.foreach { case (shuffleKey, files) =>
       try {
-        db.put(dbShuffleKey(shuffleKey), PbSerDeUtils.toPbFileInfoMap(files))
+        db.put(
+          dbShuffleKey(shuffleKey),
+          PbSerDeUtils.toPbFileInfoMap(files),
+          // K8s container might gone
+          new WriteOptions().sync(true))
         logDebug(s"Update FileInfos into DB: ${shuffleKey} -> ${files}")
       } catch {
         case exception: Exception =>
@@ -264,6 +307,29 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       partitionType: PartitionType,
       rangeReadFilter: Boolean,
       userIdentifier: UserIdentifier): FileWriter = {
+    createWriter(
+      appId,
+      shuffleId,
+      location,
+      splitThreshold,
+      splitMode,
+      partitionType,
+      rangeReadFilter,
+      userIdentifier,
+      true)
+  }
+
+  @throws[IOException]
+  def createWriter(
+      appId: String,
+      shuffleId: Int,
+      location: PartitionLocation,
+      splitThreshold: Long,
+      splitMode: PartitionSplitMode,
+      partitionType: PartitionType,
+      rangeReadFilter: Boolean,
+      userIdentifier: UserIdentifier,
+      partitionSplitEnabled: Boolean): FileWriter = {
     if (healthyWorkingDirs().size <= 0 && !hasHDFSStorage) {
       throw new IOException("No available working dirs!")
     }
@@ -289,9 +355,14 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       if (dirs.isEmpty) {
         val shuffleDir =
           new Path(new Path(hdfsDir, conf.workerWorkingDir), s"$appId/$shuffleId")
-        FileSystem.mkdirs(StorageManager.hadoopFs, shuffleDir, hdfsPermission)
         val fileInfo =
-          new FileInfo(new Path(shuffleDir, fileName).toString, userIdentifier, partitionType)
+          new FileInfo(
+            new Path(shuffleDir, fileName).toString,
+            userIdentifier,
+            partitionType,
+            partitionSplitEnabled)
+        fileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
+        FileSystem.mkdirs(StorageManager.hadoopFs, shuffleDir, hdfsPermission)
         val hdfsWriter = partitionType match {
           case PartitionType.MAP => new MapPartitionFileWriter(
               fileInfo,
@@ -313,8 +384,10 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
               rangeReadFilter)
           case _ => throw new UnsupportedOperationException(s"Not support $partitionType yet")
         }
-
-        fileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
+        if (workerGracefulShutdown) {
+          hdfsWriter.setStorageManager(this)
+          hdfsWriter.setShuffleKey(shuffleKey)
+        }
         hdfsWriters.put(fileInfo.getFilePath, hdfsWriter)
         return hdfsWriter
       } else {
@@ -323,6 +396,14 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         val shuffleDir = new File(dir, s"$appId/$shuffleId")
         val file = new File(shuffleDir, fileName)
         try {
+          val fileInfo =
+            new FileInfo(
+              file.getAbsolutePath,
+              userIdentifier,
+              partitionType,
+              partitionSplitEnabled)
+          fileInfo.setMountPoint(mountPoint)
+          fileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
           shuffleDir.mkdirs()
           if (file.exists()) {
             throw new FileAlreadyExistsException(
@@ -334,8 +415,6 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
                 s"Create shuffle data file ${file.getAbsolutePath} failed!")
             }
           }
-          val fileInfo = new FileInfo(file.getAbsolutePath, userIdentifier, partitionType)
-          fileInfo.setMountPoint(mountPoint)
           val fileWriter = partitionType match {
             case PartitionType.MAP => new MapPartitionFileWriter(
                 fileInfo,
@@ -357,10 +436,13 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
                 rangeReadFilter)
             case _ => throw new UnsupportedOperationException(s"Not support $partitionType yet")
           }
+          if (workerGracefulShutdown) {
+            fileWriter.setStorageManager(this)
+            fileWriter.setShuffleKey(shuffleKey)
+          }
           deviceMonitor.registerFileWriter(fileWriter)
           val map = workingDirWriters.computeIfAbsent(dir, workingDirWriterListFunc)
           map.put(fileInfo.getFilePath, fileWriter)
-          fileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
           location.getStorageInfo.setMountPoint(mountPoint)
           logDebug(s"location $location set disk hint to ${location.getStorageInfo} ")
           return fileWriter
@@ -456,7 +538,9 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     isHdfsExpired
   }
 
-  def cleanupExpiredShuffleKey(expiredShuffleKeys: util.HashSet[String]): Unit = {
+  def cleanupExpiredShuffleKey(
+      expiredShuffleKeys: util.HashSet[String],
+      cleanDB: Boolean = true): Unit = {
     expiredShuffleKeys.asScala.foreach { shuffleKey =>
       logInfo(s"Cleanup expired shuffle $shuffleKey.")
       if (fileInfos.containsKey(shuffleKey)) {
@@ -486,6 +570,12 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
             case e: Exception => logWarning("Clean expired HDFS shuffle failed.", e)
           }
         }
+        if (workerGracefulShutdown) {
+          committedFileInfos.remove(shuffleKey)
+          if (cleanDB) {
+            db.delete(dbShuffleKey(shuffleKey))
+          }
+        }
       }
     }
   }
@@ -510,20 +600,21 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     TimeUnit.MINUTES)
 
   private def cleanupExpiredAppDirs(): Unit = {
+    val diskInfoAndAppDirs = disksSnapshot()
+      .filter(_.status != DiskStatus.IO_HANG)
+      .map { case diskInfo =>
+        (diskInfo, diskInfo.dirs.filter(_.exists).flatMap(_.listFiles()))
+      }
     val appIds = shuffleKeySet().asScala.map(key => Utils.splitShuffleKey(key)._1)
-    disksSnapshot().filter(_.status != DiskStatus.IO_HANG).foreach { diskInfo =>
-      diskInfo.dirs.foreach {
-        case workingDir if workingDir.exists() =>
-          workingDir.listFiles().foreach { appDir =>
-            // Don't delete shuffleKey's data that exist correct shuffle file info.
-            if (!appIds.contains(appDir.getName)) {
-              val threadPool = diskOperators.get(diskInfo.mountPoint)
-              deleteDirectory(appDir, threadPool)
-              logInfo(s"Delete expired app dir $appDir.")
-            }
-          }
-        // workingDir not exist when initializing worker on new disk
-        case _ => // do nothing
+
+    diskInfoAndAppDirs.foreach { case (diskInfo, appDirs) =>
+      appDirs.foreach { appDir =>
+        // Don't delete shuffleKey's data that exist correct shuffle file info.
+        if (!appIds.contains(appDir.getName)) {
+          val threadPool = diskOperators.get(diskInfo.mountPoint)
+          deleteDirectory(appDir, threadPool)
+          logInfo(s"Delete expired app dir $appDir.")
+        }
       }
     }
   }
@@ -603,19 +694,26 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
     false
   }
 
-  def close(): Unit = {
+  def close(exitKind: Int): Unit = {
     if (db != null) {
-      try {
-        updateFileInfosInDB()
-        db.close()
-      } catch {
-        case exception: Exception =>
-          logError("Store recover data to LevelDB failed.", exception)
+      if (exitKind == CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN) {
+        try {
+          saveAllCommittedFileInfosToDB()
+          db.close()
+        } catch {
+          case exception: Exception =>
+            logError("Store recover data to LevelDB failed.", exception)
+        }
+      } else {
+        if (db != null) {
+          db.close()
+          new File(conf.workerGracefulShutdownRecoverPath, RECOVERY_FILE_NAME).delete()
+        }
       }
     }
     if (null != diskOperators) {
-      if (!conf.workerGracefulShutdown) {
-        cleanupExpiredShuffleKey(shuffleKeySet())
+      if (exitKind != CelebornExitKind.WORKER_GRACEFUL_SHUTDOWN) {
+        cleanupExpiredShuffleKey(shuffleKeySet(), false)
       }
       ThreadUtils.parmap(
         diskOperators.asScala.toMap,
@@ -730,6 +828,18 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         }
     }
   }
+
+  def notifyFileInfoCommitted(
+      shuffleKey: String,
+      fileName: String,
+      fileInfo: FileInfo): Unit = {
+    committedFileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
+  }
+
+  def getActiveShuffleSize(): Long = {
+    fileInfos.values().asScala.map(_.values().asScala.map(_.getBytesFlushed).sum).sum
+  }
+
 }
 
 object StorageManager {
